#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace Kiran {
namespace SystemDaemon {

class AccountsTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase FindUserById_pack(
        guint64 arg_uid) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint64> params =
            Glib::Variant<guint64>::create(arg_uid);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase FindUserByName_pack(
        const Glib::ustring & arg_name) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_name);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase FindUserByAuthData_pack(
        gint32 arg_mode,
        const Glib::ustring & arg_data_id) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint32> mode_param =
            Glib::Variant<gint32>::create(arg_mode);
        params.push_back(mode_param);

        Glib::Variant<Glib::ustring> data_id_param =
            Glib::Variant<Glib::ustring>::create(arg_data_id);
        params.push_back(data_id_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase CreateUser_pack(
        const Glib::ustring & arg_name,
        const Glib::ustring & arg_realname,
        gint32 arg_account_type,
        gint64 arg_uid) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> name_param =
            Glib::Variant<Glib::ustring>::create(arg_name);
        params.push_back(name_param);

        Glib::Variant<Glib::ustring> realname_param =
            Glib::Variant<Glib::ustring>::create(arg_realname);
        params.push_back(realname_param);

        Glib::Variant<gint32> account_type_param =
            Glib::Variant<gint32>::create(arg_account_type);
        params.push_back(account_type_param);

        Glib::Variant<gint64> uid_param =
            Glib::Variant<gint64>::create(arg_uid);
        params.push_back(uid_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase DeleteUser_pack(
        guint64 arg_uid,
        bool arg_remove_files) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<guint64> uid_param =
            Glib::Variant<guint64>::create(arg_uid);
        params.push_back(uid_param);

        Glib::Variant<bool> remove_files_param =
            Glib::Variant<bool>::create(arg_remove_files);
        params.push_back(remove_files_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // SystemDaemon
} // Kiran


