static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="com.kylinsec.Kiran.SystemDaemon.Accounts">
        <method name="GetNonSystemUsers">
            <arg name="users" direction="out" type="ao">
                <description>Object paths of non system users</description>
            </arg>
        </method>

        <method name="FindUserById">
            <arg name="uid" direction="in" type="t">
                <description>The uid to look up</description>
            </arg>
            <arg name="user" direction="out" type="o">
                <description>Object path of user</description>
            </arg>
        </method>

        <method name="FindUserByName">
            <arg name="name" direction="in" type="s">
                <description>The username to look up</description>
            </arg>
            <arg name="user" direction="out" type="o">
                <description>Object path of user</description>
            </arg>
        </method>

        <method name="FindUserByAuthData">
            <arg name="mode" direction="in" type="i">
                <description>The authentication mode which could be fingerprint, face, etc. Refer to AccountsAuthMode in accounts-i.h</description>
            </arg>
            <arg name="data_id" direction="in" type="s">
                <description>The identity of the collected data for the authentication item.</description>
            </arg>
            <arg name="user" direction="out" type="o">
                <description>Object path of the new user.</description>
            </arg>
        </method>

        <method name="CreateUser">
            <arg name="name" direction="in" type="s">
                <description>The username for the new user</description>
            </arg>
            <arg name="realname" direction="in" type="s">
                <description>The real name for the new user</description>
            </arg>
            <arg name="account_type" direction="in" type="i">
                <description>The account type, encoded as an integer</description>
            </arg>
            <arg name="uid" direction="in" type="x">
                <description>The user uid. If uid is less than zero, then auto to allocate one.</description>
            </arg>
            <arg name="user" direction="out" type="o">
                <description>Object path of the new user</description>
            </arg>
        </method>

        <method name="DeleteUser">
            <arg name="uid" direction="in" type="t">
                <description>The uid to delete</description>
            </arg>
            <arg name="remove_files" direction="in" type="b">
                <description>Whether to remove the users files</description>
            </arg>
        </method>

        <!--<method name="SetGlobalAuthModes">
            <arg name="auth_modes" direction="in" type="i">
                <description>Bitwise OR the enabled authorization mode.</description>
            </arg>>
            <description>Set the global enabled authentication modes. It will reset to all users using the auth_modes
            and also use the auth_modes when user is added.</description>
        </method>-->

        <signal name="UserAdded">
            <arg name="user" type="o">
                <description>Object path of the user that was added.</description>
            </arg>
        </signal>

        <signal name="UserDeleted">
            <arg name="user" type="o">
                <description>Object path of the user that was deleted.</description>
            </arg>
        </signal>

    </interface>
</node>)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/accounts_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

Kiran::SystemDaemon::AccountsStub::AccountsStub():
    m_interfaceName("com.kylinsec.Kiran.SystemDaemon.Accounts")
{
    UserAdded_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &AccountsStub::UserAdded_emitter),
            std::vector<Glib::ustring>({""})) );
    UserAdded_selectiveSignal.connect(sigc::mem_fun(this, &AccountsStub::UserAdded_emitter));
    UserDeleted_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &AccountsStub::UserDeleted_emitter),
            std::vector<Glib::ustring>({""})) );
    UserDeleted_selectiveSignal.connect(sigc::mem_fun(this, &AccountsStub::UserDeleted_emitter));
}

Kiran::SystemDaemon::AccountsStub::~AccountsStub()
{
    unregister_object();
}

guint Kiran::SystemDaemon::AccountsStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &AccountsStub::on_method_call),
            sigc::mem_fun(this, &AccountsStub::on_interface_get_property),
            sigc::mem_fun(this, &AccountsStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("com.kylinsec.Kiran.SystemDaemon.Accounts"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void Kiran::SystemDaemon::AccountsStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void Kiran::SystemDaemon::AccountsStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("GetNonSystemUsers") == 0) {
        MethodInvocation methodInvocation(invocation);
        GetNonSystemUsers(
            methodInvocation);
    }

    if (method_name.compare("FindUserById") == 0) {
        Glib::Variant<guint64> base_uid;
        parameters.get_child(base_uid, 0);
        guint64 p_uid = specialGetter(base_uid);

        MethodInvocation methodInvocation(invocation);
        FindUserById(
            (p_uid),
            methodInvocation);
    }

    if (method_name.compare("FindUserByName") == 0) {
        Glib::Variant<Glib::ustring> base_name;
        parameters.get_child(base_name, 0);
        Glib::ustring p_name = specialGetter(base_name);

        MethodInvocation methodInvocation(invocation);
        FindUserByName(
            (p_name),
            methodInvocation);
    }

    if (method_name.compare("FindUserByAuthData") == 0) {
        Glib::Variant<gint32> base_mode;
        parameters.get_child(base_mode, 0);
        gint32 p_mode = specialGetter(base_mode);

        Glib::Variant<Glib::ustring> base_data_id;
        parameters.get_child(base_data_id, 1);
        Glib::ustring p_data_id = specialGetter(base_data_id);

        MethodInvocation methodInvocation(invocation);
        FindUserByAuthData(
            (p_mode),
            (p_data_id),
            methodInvocation);
    }

    if (method_name.compare("CreateUser") == 0) {
        Glib::Variant<Glib::ustring> base_name;
        parameters.get_child(base_name, 0);
        Glib::ustring p_name = specialGetter(base_name);

        Glib::Variant<Glib::ustring> base_realname;
        parameters.get_child(base_realname, 1);
        Glib::ustring p_realname = specialGetter(base_realname);

        Glib::Variant<gint32> base_account_type;
        parameters.get_child(base_account_type, 2);
        gint32 p_account_type = specialGetter(base_account_type);

        Glib::Variant<gint64> base_uid;
        parameters.get_child(base_uid, 3);
        gint64 p_uid = specialGetter(base_uid);

        MethodInvocation methodInvocation(invocation);
        CreateUser(
            (p_name),
            (p_realname),
            (p_account_type),
            (p_uid),
            methodInvocation);
    }

    if (method_name.compare("DeleteUser") == 0) {
        Glib::Variant<guint64> base_uid;
        parameters.get_child(base_uid, 0);
        guint64 p_uid = specialGetter(base_uid);

        Glib::Variant<bool> base_remove_files;
        parameters.get_child(base_remove_files, 1);
        bool p_remove_files = specialGetter(base_remove_files);

        MethodInvocation methodInvocation(invocation);
        DeleteUser(
            (p_uid),
            (p_remove_files),
            methodInvocation);
    }

}

void Kiran::SystemDaemon::AccountsStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

}

bool Kiran::SystemDaemon::AccountsStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    return true;
}

void Kiran::SystemDaemon::AccountsStub::UserAdded_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,const Glib::DBusObjectPathString & user)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<Glib::DBusObjectPathString>::create((user)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "com.kylinsec.Kiran.SystemDaemon.Accounts",
                    "UserAdded",
                    bus_name,
                    params);
        }
    }
}

void Kiran::SystemDaemon::AccountsStub::UserDeleted_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,const Glib::DBusObjectPathString & user)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<Glib::DBusObjectPathString>::create((user)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "com.kylinsec.Kiran.SystemDaemon.Accounts",
                    "UserDeleted",
                    bus_name,
                    params);
        }
    }
}


bool Kiran::SystemDaemon::AccountsStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
