#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/accounts_dbus_common.h"

namespace Kiran {
namespace SystemDaemon {

class AccountsStub : public sigc::trackable {
public:
    AccountsStub();
    virtual ~AccountsStub();

    AccountsStub(const AccountsStub &other) = delete;
    AccountsStub(AccountsStub &&other) = delete;
    AccountsStub &operator=(const AccountsStub &other) = delete;
    AccountsStub &operator=(AccountsStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

protected:
    virtual void GetNonSystemUsers(
        MethodInvocation &invocation) = 0;
    virtual void FindUserById(
        guint64 uid,
        MethodInvocation &invocation) = 0;
    virtual void FindUserByName(
        const Glib::ustring & name,
        MethodInvocation &invocation) = 0;
    virtual void FindUserByAuthData(
        gint32 mode,
        const Glib::ustring & data_id,
        MethodInvocation &invocation) = 0;
    virtual void CreateUser(
        const Glib::ustring & name,
        const Glib::ustring & realname,
        gint32 account_type,
        gint64 uid,
        MethodInvocation &invocation) = 0;
    virtual void DeleteUser(
        guint64 uid,
        bool remove_files,
        MethodInvocation &invocation) = 0;

    void UserAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> UserAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> UserAdded_selectiveSignal;

    void UserDeleted_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> UserDeleted_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> UserDeleted_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class AccountsStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret(const std::vector<Glib::DBusObjectPathString> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::vector<Glib::DBusObjectPathString>> var0 =
            Glib::Variant<std::vector<Glib::DBusObjectPathString>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const Glib::DBusObjectPathString & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::VariantStringBase var0;
        Glib::VariantStringBase::create_object_path(var0, p0.c_str());
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SystemDaemon
} // Kiran

