#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/appearance_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class AppearanceProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<AppearanceProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<AppearanceProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void GetThemes(
        gint32 type,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetThemes_finish (
        Glib::ustring &theme_json,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    Glib::ustring
    GetThemes_sync(
        gint32 type,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetTheme(
        gint32 type,
        const Glib::ustring & theme_name,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetTheme_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetTheme_sync(
        gint32 type,        const Glib::ustring & theme_name,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetTheme(
        gint32 type,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetTheme_finish (
        Glib::ustring &theme_name,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    Glib::ustring
    GetTheme_sync(
        gint32 type,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetFont(
        gint32 type,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetFont_finish (
        Glib::ustring &font,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    Glib::ustring
    GetFont_sync(
        gint32 type,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetFont(
        gint32 type,
        const Glib::ustring & font,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetFont_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetFont_sync(
        gint32 type,        const Glib::ustring & font,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetDesktopBackground(
        const Glib::ustring & desktop_background,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetDesktopBackground_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetDesktopBackground_sync(
        const Glib::ustring & desktop_background,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetLockScreenBackground(
        const Glib::ustring & lock_screen_background,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetLockScreenBackground_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetLockScreenBackground_sync(
        const Glib::ustring & lock_screen_background,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    Glib::ustring desktop_background_get(bool *ok = nullptr);
    sigc::signal<void> &desktop_background_changed() {
        return m_desktop_background_changed;
    }

    Glib::ustring lock_screen_background_get(bool *ok = nullptr);
    sigc::signal<void> &lock_screen_background_changed() {
        return m_lock_screen_background_changed;
    }

    sigc::signal<void, gint32, Glib::ustring > ThemeChanged_signal;

    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    AppearanceProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_desktop_background_changed;
    sigc::signal<void> m_lock_screen_background_changed;
};

}// SessionDaemon
}// Kiran

