#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/appearance_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class AppearanceStub : public sigc::trackable {
public:
    AppearanceStub();
    virtual ~AppearanceStub();

    AppearanceStub(const AppearanceStub &other) = delete;
    AppearanceStub(AppearanceStub &&other) = delete;
    AppearanceStub &operator=(const AppearanceStub &other) = delete;
    AppearanceStub &operator=(AppearanceStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool desktop_background_set(const Glib::ustring & value);
    bool lock_screen_background_set(const Glib::ustring & value);
protected:
    virtual void GetThemes(
        gint32 type,
        MethodInvocation &invocation) = 0;
    virtual void SetTheme(
        gint32 type,
        const Glib::ustring & theme_name,
        MethodInvocation &invocation) = 0;
    virtual void GetTheme(
        gint32 type,
        MethodInvocation &invocation) = 0;
    virtual void GetFont(
        gint32 type,
        MethodInvocation &invocation) = 0;
    virtual void SetFont(
        gint32 type,
        const Glib::ustring & font,
        MethodInvocation &invocation) = 0;
    virtual void SetDesktopBackground(
        const Glib::ustring & desktop_background,
        MethodInvocation &invocation) = 0;
    virtual void SetLockScreenBackground(
        const Glib::ustring & lock_screen_background,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool desktop_background_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring desktop_background_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool lock_screen_background_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring lock_screen_background_get() = 0;

    void ThemeChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, gint32, const Glib::ustring &);
    sigc::signal<void, gint32, const Glib::ustring &> ThemeChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, gint32, const Glib::ustring &> ThemeChanged_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class AppearanceStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret(const Glib::ustring & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<Glib::ustring> var0 =
            Glib::Variant<Glib::ustring>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionDaemon
} // Kiran

