#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/audio_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class AudioStub : public sigc::trackable {
public:
    AudioStub();
    virtual ~AudioStub();

    AudioStub(const AudioStub &other) = delete;
    AudioStub(AudioStub &&other) = delete;
    AudioStub &operator=(const AudioStub &other) = delete;
    AudioStub &operator=(AudioStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool state_set(guint32 value);
protected:
    virtual void GetDefaultSink(
        MethodInvocation &invocation) = 0;
    virtual void SetDefaultSink(
        guint32 sink_index,
        MethodInvocation &invocation) = 0;
    virtual void GetSinks(
        MethodInvocation &invocation) = 0;
    virtual void GetSink(
        guint32 index,
        MethodInvocation &invocation) = 0;
    virtual void GetDefaultSource(
        MethodInvocation &invocation) = 0;
    virtual void SetDefaultSource(
        guint32 source_index,
        MethodInvocation &invocation) = 0;
    virtual void GetSources(
        MethodInvocation &invocation) = 0;
    virtual void GetSource(
        guint32 index,
        MethodInvocation &invocation) = 0;
    virtual void GetSinkInputs(
        MethodInvocation &invocation) = 0;
    virtual void GetSinkInput(
        guint32 index,
        MethodInvocation &invocation) = 0;
    virtual void GetSourceOutputs(
        MethodInvocation &invocation) = 0;
    virtual void GetSourceOutput(
        guint32 index,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool state_setHandler(guint32 value) = 0;
    virtual guint32 state_get() = 0;

    void DefaultSinkChange_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> DefaultSinkChange_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> DefaultSinkChange_selectiveSignal;

    void DefaultSourceChange_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> DefaultSourceChange_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> DefaultSourceChange_selectiveSignal;

    void SinkAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SinkAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SinkAdded_selectiveSignal;

    void SinkDelete_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SinkDelete_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SinkDelete_selectiveSignal;

    void SourceAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SourceAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SourceAdded_selectiveSignal;

    void SourceDelete_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SourceDelete_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SourceDelete_selectiveSignal;

    void SinkInputAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SinkInputAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SinkInputAdded_selectiveSignal;

    void SinkInputDelete_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SinkInputDelete_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SinkInputDelete_selectiveSignal;

    void SourceOutputAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SourceOutputAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SourceOutputAdded_selectiveSignal;

    void SourceOutputDelete_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> SourceOutputDelete_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> SourceOutputDelete_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class AudioStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret(const Glib::ustring & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<Glib::ustring> var0 =
            Glib::Variant<Glib::ustring>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const std::vector<Glib::ustring> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::vector<Glib::ustring>> var0 =
            Glib::Variant<std::vector<Glib::ustring>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionDaemon
} // Kiran

