#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/audio_device_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {
namespace Audio {

class DeviceProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<DeviceProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<DeviceProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void SetActivePort(
        const Glib::ustring & name,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetActivePort_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetActivePort_sync(
        const Glib::ustring & name,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetPorts(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetPorts_finish (
        Glib::ustring &ports,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    Glib::ustring
    GetPorts_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetVolume(
        double volume,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetVolume_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetVolume_sync(
        double volume,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetBalance(
        double balance,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetBalance_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetBalance_sync(
        double balance,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetFade(
        double fade,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetFade_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetFade_sync(
        double fade,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetMute(
        bool mute,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetMute_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetMute_sync(
        bool mute,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    guint32 index_get(bool *ok = nullptr);
    sigc::signal<void> &index_changed() {
        return m_index_changed;
    }

    Glib::ustring name_get(bool *ok = nullptr);
    sigc::signal<void> &name_changed() {
        return m_name_changed;
    }

    bool mute_get(bool *ok = nullptr);
    sigc::signal<void> &mute_changed() {
        return m_mute_changed;
    }

    double volume_get(bool *ok = nullptr);
    sigc::signal<void> &volume_changed() {
        return m_volume_changed;
    }

    double balance_get(bool *ok = nullptr);
    sigc::signal<void> &balance_changed() {
        return m_balance_changed;
    }

    double fade_get(bool *ok = nullptr);
    sigc::signal<void> &fade_changed() {
        return m_fade_changed;
    }

    double base_volume_get(bool *ok = nullptr);
    sigc::signal<void> &base_volume_changed() {
        return m_base_volume_changed;
    }

    guint32 card_index_get(bool *ok = nullptr);
    sigc::signal<void> &card_index_changed() {
        return m_card_index_changed;
    }

    Glib::ustring active_port_get(bool *ok = nullptr);
    sigc::signal<void> &active_port_changed() {
        return m_active_port_changed;
    }

    guint32 state_get(bool *ok = nullptr);
    sigc::signal<void> &state_changed() {
        return m_state_changed;
    }


    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    DeviceProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_index_changed;
    sigc::signal<void> m_name_changed;
    sigc::signal<void> m_mute_changed;
    sigc::signal<void> m_volume_changed;
    sigc::signal<void> m_balance_changed;
    sigc::signal<void> m_fade_changed;
    sigc::signal<void> m_base_volume_changed;
    sigc::signal<void> m_card_index_changed;
    sigc::signal<void> m_active_port_changed;
    sigc::signal<void> m_state_changed;
};

}// Audio
}// SessionDaemon
}// Kiran

