#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/audio_device_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {
namespace Audio {

class DeviceStub : public sigc::trackable {
public:
    DeviceStub();
    virtual ~DeviceStub();

    DeviceStub(const DeviceStub &other) = delete;
    DeviceStub(DeviceStub &&other) = delete;
    DeviceStub &operator=(const DeviceStub &other) = delete;
    DeviceStub &operator=(DeviceStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool index_set(guint32 value);
    bool name_set(const Glib::ustring & value);
    bool mute_set(bool value);
    bool volume_set(double value);
    bool balance_set(double value);
    bool fade_set(double value);
    bool base_volume_set(double value);
    bool card_index_set(guint32 value);
    bool active_port_set(const Glib::ustring & value);
    bool state_set(guint32 value);
protected:
    virtual void SetActivePort(
        const Glib::ustring & name,
        MethodInvocation &invocation) = 0;
    virtual void GetPorts(
        MethodInvocation &invocation) = 0;
    virtual void SetVolume(
        double volume,
        MethodInvocation &invocation) = 0;
    virtual void SetBalance(
        double balance,
        MethodInvocation &invocation) = 0;
    virtual void SetFade(
        double fade,
        MethodInvocation &invocation) = 0;
    virtual void SetMute(
        bool mute,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool index_setHandler(guint32 value) = 0;
    virtual guint32 index_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool name_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring name_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool mute_setHandler(bool value) = 0;
    virtual bool mute_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool volume_setHandler(double value) = 0;
    virtual double volume_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool balance_setHandler(double value) = 0;
    virtual double balance_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool fade_setHandler(double value) = 0;
    virtual double fade_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool base_volume_setHandler(double value) = 0;
    virtual double base_volume_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool card_index_setHandler(guint32 value) = 0;
    virtual guint32 card_index_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool active_port_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring active_port_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool state_setHandler(guint32 value) = 0;
    virtual guint32 state_get() = 0;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class DeviceStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const Glib::ustring & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<Glib::ustring> var0 =
            Glib::Variant<Glib::ustring>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // Audio
} // SessionDaemon
} // Kiran

