#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace Kiran {
namespace SessionDaemon {

class BluetoothTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase GetDevices_pack(
        const Glib::DBusObjectPathString & arg_adapter) {
        Glib::VariantContainerBase base;
        Glib::VariantStringBase params;
        Glib::VariantStringBase::create_object_path(params, arg_adapter.c_str());
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase FeedPinCode_pack(
        const Glib::DBusObjectPathString & arg_device,
        bool arg_accept,
        const Glib::ustring & arg_pincode) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::VariantStringBase device_param;
        Glib::VariantStringBase::create_object_path(device_param, arg_device.c_str());
        params.push_back(device_param);

        Glib::Variant<bool> accept_param =
            Glib::Variant<bool>::create(arg_accept);
        params.push_back(accept_param);

        Glib::Variant<Glib::ustring> pincode_param =
            Glib::Variant<Glib::ustring>::create(arg_pincode);
        params.push_back(pincode_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase FeedPasskey_pack(
        const Glib::DBusObjectPathString & arg_device,
        bool arg_accept,
        guint32 arg_passkey) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::VariantStringBase device_param;
        Glib::VariantStringBase::create_object_path(device_param, arg_device.c_str());
        params.push_back(device_param);

        Glib::Variant<bool> accept_param =
            Glib::Variant<bool>::create(arg_accept);
        params.push_back(accept_param);

        Glib::Variant<guint32> passkey_param =
            Glib::Variant<guint32>::create(arg_passkey);
        params.push_back(passkey_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase Confirm_pack(
        const Glib::DBusObjectPathString & arg_device,
        bool arg_accept) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::VariantStringBase device_param;
        Glib::VariantStringBase::create_object_path(device_param, arg_device.c_str());
        params.push_back(device_param);

        Glib::Variant<bool> accept_param =
            Glib::Variant<bool>::create(arg_accept);
        params.push_back(accept_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // SessionDaemon
} // Kiran


