#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/bluetooth_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class BluetoothProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<BluetoothProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<BluetoothProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void GetAdapters(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetAdapters_finish (
        std::vector<Glib::ustring> &adapters,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    std::vector<Glib::ustring>
    GetAdapters_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetDevices(
        const Glib::DBusObjectPathString & adapter,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetDevices_finish (
        std::vector<Glib::ustring> &devices,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    std::vector<Glib::ustring>
    GetDevices_sync(
        const Glib::DBusObjectPathString & adapter,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void FeedPinCode(
        const Glib::DBusObjectPathString & device,
        bool accept,
        const Glib::ustring & pincode,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void FeedPinCode_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    FeedPinCode_sync(
        const Glib::DBusObjectPathString & device,        bool accept,        const Glib::ustring & pincode,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void FeedPasskey(
        const Glib::DBusObjectPathString & device,
        bool accept,
        guint32 passkey,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void FeedPasskey_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    FeedPasskey_sync(
        const Glib::DBusObjectPathString & device,        bool accept,        guint32 passkey,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void Confirm(
        const Glib::DBusObjectPathString & device,
        bool accept,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void Confirm_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    Confirm_sync(
        const Glib::DBusObjectPathString & device,        bool accept,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    sigc::signal<void, Glib::DBusObjectPathString > AdapterAdded_signal;
    sigc::signal<void, Glib::DBusObjectPathString > AdapterRemoved_signal;
    sigc::signal<void, Glib::DBusObjectPathString > DeviceAdded_signal;
    sigc::signal<void, Glib::DBusObjectPathString > DeviceRemove_signal;
    sigc::signal<void, Glib::DBusObjectPathString > RequestPinCode_signal;
    sigc::signal<void, Glib::DBusObjectPathString, Glib::ustring > DisplayPinCode_signal;
    sigc::signal<void, Glib::DBusObjectPathString > RequestPasskey_signal;
    sigc::signal<void, Glib::DBusObjectPathString, guint32, guint16 > DisplayPasskey_signal;
    sigc::signal<void, Glib::DBusObjectPathString, guint32 > RequestConfirmation_signal;
    sigc::signal<void, Glib::DBusObjectPathString > RequestAuthorization_signal;
    sigc::signal<void, Glib::DBusObjectPathString, Glib::ustring > AuthorizeService_signal;
    sigc::signal<void, Glib::DBusObjectPathString > Cancel_signal;

    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    BluetoothProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

};

}// SessionDaemon
}// Kiran

