#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/bluetooth_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class BluetoothStub : public sigc::trackable {
public:
    BluetoothStub();
    virtual ~BluetoothStub();

    BluetoothStub(const BluetoothStub &other) = delete;
    BluetoothStub(BluetoothStub &&other) = delete;
    BluetoothStub &operator=(const BluetoothStub &other) = delete;
    BluetoothStub &operator=(BluetoothStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

protected:
    virtual void GetAdapters(
        MethodInvocation &invocation) = 0;
    virtual void GetDevices(
        const Glib::DBusObjectPathString & adapter,
        MethodInvocation &invocation) = 0;
    virtual void FeedPinCode(
        const Glib::DBusObjectPathString & device,
        bool accept,
        const Glib::ustring & pincode,
        MethodInvocation &invocation) = 0;
    virtual void FeedPasskey(
        const Glib::DBusObjectPathString & device,
        bool accept,
        guint32 passkey,
        MethodInvocation &invocation) = 0;
    virtual void Confirm(
        const Glib::DBusObjectPathString & device,
        bool accept,
        MethodInvocation &invocation) = 0;

    void AdapterAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> AdapterAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> AdapterAdded_selectiveSignal;

    void AdapterRemoved_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> AdapterRemoved_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> AdapterRemoved_selectiveSignal;

    void DeviceAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> DeviceAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> DeviceAdded_selectiveSignal;

    void DeviceRemove_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> DeviceRemove_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> DeviceRemove_selectiveSignal;

    void RequestPinCode_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> RequestPinCode_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> RequestPinCode_selectiveSignal;

    void DisplayPinCode_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &, const Glib::ustring &);
    sigc::signal<void, const Glib::DBusObjectPathString &, const Glib::ustring &> DisplayPinCode_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &, const Glib::ustring &> DisplayPinCode_selectiveSignal;

    void RequestPasskey_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> RequestPasskey_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> RequestPasskey_selectiveSignal;

    void DisplayPasskey_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &, guint32, guint16);
    sigc::signal<void, const Glib::DBusObjectPathString &, guint32, guint16> DisplayPasskey_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &, guint32, guint16> DisplayPasskey_selectiveSignal;

    void RequestConfirmation_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &, guint32);
    sigc::signal<void, const Glib::DBusObjectPathString &, guint32> RequestConfirmation_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &, guint32> RequestConfirmation_selectiveSignal;

    void RequestAuthorization_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> RequestAuthorization_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> RequestAuthorization_selectiveSignal;

    void AuthorizeService_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &, const Glib::ustring &);
    sigc::signal<void, const Glib::DBusObjectPathString &, const Glib::ustring &> AuthorizeService_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &, const Glib::ustring &> AuthorizeService_selectiveSignal;

    void Cancel_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &);
    sigc::signal<void, const Glib::DBusObjectPathString &> Cancel_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &> Cancel_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class BluetoothStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret(const std::vector<Glib::ustring> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::vector<Glib::ustring>> var0 =
            Glib::Variant<std::vector<Glib::ustring>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionDaemon
} // Kiran

