#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace bluez {

class Agent1TypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase RequestPinCode_pack(
        const Glib::DBusObjectPathString & arg_device) {
        Glib::VariantContainerBase base;
        Glib::VariantStringBase params;
        Glib::VariantStringBase::create_object_path(params, arg_device.c_str());
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase DisplayPinCode_pack(
        const Glib::DBusObjectPathString & arg_device,
        const Glib::ustring & arg_pincode) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::VariantStringBase device_param;
        Glib::VariantStringBase::create_object_path(device_param, arg_device.c_str());
        params.push_back(device_param);

        Glib::Variant<Glib::ustring> pincode_param =
            Glib::Variant<Glib::ustring>::create(arg_pincode);
        params.push_back(pincode_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase RequestPasskey_pack(
        const Glib::DBusObjectPathString & arg_device) {
        Glib::VariantContainerBase base;
        Glib::VariantStringBase params;
        Glib::VariantStringBase::create_object_path(params, arg_device.c_str());
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase DisplayPasskey_pack(
        const Glib::DBusObjectPathString & arg_device,
        guint32 arg_passkey,
        guint16 arg_entered) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::VariantStringBase device_param;
        Glib::VariantStringBase::create_object_path(device_param, arg_device.c_str());
        params.push_back(device_param);

        Glib::Variant<guint32> passkey_param =
            Glib::Variant<guint32>::create(arg_passkey);
        params.push_back(passkey_param);

        Glib::Variant<guint16> entered_param =
            Glib::Variant<guint16>::create(arg_entered);
        params.push_back(entered_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase RequestConfirmation_pack(
        const Glib::DBusObjectPathString & arg_device,
        guint32 arg_passkey) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::VariantStringBase device_param;
        Glib::VariantStringBase::create_object_path(device_param, arg_device.c_str());
        params.push_back(device_param);

        Glib::Variant<guint32> passkey_param =
            Glib::Variant<guint32>::create(arg_passkey);
        params.push_back(passkey_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase RequestAuthorization_pack(
        const Glib::DBusObjectPathString & arg_device) {
        Glib::VariantContainerBase base;
        Glib::VariantStringBase params;
        Glib::VariantStringBase::create_object_path(params, arg_device.c_str());
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase AuthorizeService_pack(
        const Glib::DBusObjectPathString & arg_device,
        const Glib::ustring & arg_uuid) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::VariantStringBase device_param;
        Glib::VariantStringBase::create_object_path(device_param, arg_device.c_str());
        params.push_back(device_param);

        Glib::Variant<Glib::ustring> uuid_param =
            Glib::Variant<Glib::ustring>::create(arg_uuid);
        params.push_back(uuid_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // bluez


