static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="org.bluez.Agent1">

        <method name="Release">
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called when the service daemon unregisters the agent.
                    An agent can use it to do cleanup tasks. There is no need to unregister the agent,
                    because when this method gets called it has already been unregistered.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="RequestPinCode">
            <arg type="o" name="device" direction="in">
                <doc:doc>
                    <doc:summary>The device object path.</doc:summary>
                </doc:doc>
            </arg>
            <arg type="s" name="pincode" direction="out">
                <doc:doc>
                    <doc:summary>The pincode.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called when the service daemon needs to get the passkey for an authentication.
                    The return value should be a string of 1-16 characters length. The string can be alphanumeric.</doc:para>
                    <doc:para>Possible errors: org.bluez.Error.Rejected org.bluez.Error.Canceled</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="DisplayPinCode">
            <arg type="o" name="device" direction="in">
                <doc:doc>
                    <doc:summary>The device object path.</doc:summary>
                </doc:doc>
            </arg>
            <arg type="s" name="pincode" direction="in">
                <doc:doc>
                    <doc:summary>The pincode.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called when the service daemon needs to display a pincode for an authentication.
                    An empty reply should be returned. When the pincode needs no longer to be displayed, the Cancel method of
                    the agent will be called. This is used during the pairing process of keyboards	that don't support Bluetooth 
                    2.1 Secure Simple Pairing, in contrast to DisplayPasskey which is used for those that do. This method will
                    only ever be called once since older keyboards do not support typing notification. Note that the PIN will
                    always be a 6-digit number, zero-padded to 6 digits. This is for harmony with the later specification.</doc:para>
                    <doc:para>Possible errors: org.bluez.Error.Rejected org.bluez.Error.Canceled</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="RequestPasskey">
            <arg type="o" name="device" direction="in">
                <doc:doc>
                    <doc:summary>The device object path.</doc:summary>
                </doc:doc>
            </arg>
            <arg type="u" name="passkey" direction="out">
                <doc:doc>
                    <doc:summary>The passkey.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called when the service daemon needs to get the passkey for an authentication.
                    The return value should be a numeric value between 0-999999.</doc:para>
                    <doc:para>Possible errors: org.bluez.Error.Rejected org.bluez.Error.Canceled</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="DisplayPasskey">
            <arg type="o" name="device" direction="in">
                <doc:doc>
                    <doc:summary>The device object path.</doc:summary>
                </doc:doc>
            </arg>
            <arg type="u" name="passkey" direction="in">
                <doc:doc>
                    <doc:summary>The passkey.</doc:summary>
                </doc:doc>
            </arg>
            <arg type="q" name="entered" direction="in">
                <doc:doc>
                    <doc:summary>The number of already typed keys on the remote side.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called when the service daemon needs to display a passkey for an authentication.
                    The entered parameter indicates the number of already typed keys on the remote side. An empty reply should
                    be returned. When the passkey needs no longer to be displayed, the Cancel method of the agent will be called.
                    During the pairing process this method might be called multiple times to update the entered value. Note that
                    the passkey will always be a 6-digit number, so the display should be zero-padded at the start if the value
                    contains less than 6 digits.</doc:para>
                </doc:description>
            </doc:doc>
        </method>


        <method name="RequestConfirmation">
            <arg type="o" name="device" direction="in">
                <doc:doc>
                    <doc:summary>The device object path.</doc:summary>
                </doc:doc>
            </arg>
            <arg type="u" name="passkey" direction="in">
                <doc:doc>
                    <doc:summary>The passkey.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called when the service daemon needs to confirm a passkey for an authentication.
                    To confirm the value it should return an empty reply or an error in case the passkey is invalid. Note that
                    the passkey will always be a 6-digit number, so the display should be zero-padded at the start if the value
                    contains less than 6 digits.</doc:para>
                    <doc:para>Possible errors: org.bluez.Error.Rejected org.bluez.Error.Canceled</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="RequestAuthorization">
            <arg type="o" name="device" direction="in">
                <doc:doc>
                    <doc:summary>The device object path.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called to request the user to authorize an incoming pairing attempt which would
                    in other circumstances trigger the just-works model, or when the user plugged in a device that implements
                    cable pairing. In the latter case, the device would not be connected to the adapter via Bluetooth yet.</doc:para>
                    <doc:para>Possible errors: org.bluez.Error.Rejected org.bluez.Error.Canceled</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="AuthorizeService">
            <arg type="o" name="device" direction="in">
                <doc:doc>
                    <doc:summary>The device object path.</doc:summary>
                </doc:doc>
            </arg>
            <arg type="s" name="uuid" direction="in">
                <doc:doc>
                    <doc:summary>The device uuid.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called when the service daemon needs to authorize a connection/service request.</doc:para>
                    <doc:para>Possible errors: org.bluez.Error.Rejected org.bluez.Error.Canceled</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="Cancel">
            <doc:doc>
                <doc:description>
                    <doc:para>This method gets called to indicate that the agent request failed before a reply was returned.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

    </interface>
</node>
)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/bluez_agent_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

bluez::Agent1Stub::Agent1Stub():
    m_interfaceName("org.bluez.Agent1")
{
}

bluez::Agent1Stub::~Agent1Stub()
{
    unregister_object();
}

guint bluez::Agent1Stub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &Agent1Stub::on_method_call),
            sigc::mem_fun(this, &Agent1Stub::on_interface_get_property),
            sigc::mem_fun(this, &Agent1Stub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("org.bluez.Agent1"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void bluez::Agent1Stub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void bluez::Agent1Stub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("Release") == 0) {
        MethodInvocation methodInvocation(invocation);
        Release(
            methodInvocation);
    }

    if (method_name.compare("RequestPinCode") == 0) {
        Glib::Variant<Glib::DBusObjectPathString> base_device;
        parameters.get_child(base_device, 0);
        Glib::DBusObjectPathString p_device = specialGetter(base_device);

        MethodInvocation methodInvocation(invocation);
        RequestPinCode(
            (p_device),
            methodInvocation);
    }

    if (method_name.compare("DisplayPinCode") == 0) {
        Glib::Variant<Glib::DBusObjectPathString> base_device;
        parameters.get_child(base_device, 0);
        Glib::DBusObjectPathString p_device = specialGetter(base_device);

        Glib::Variant<Glib::ustring> base_pincode;
        parameters.get_child(base_pincode, 1);
        Glib::ustring p_pincode = specialGetter(base_pincode);

        MethodInvocation methodInvocation(invocation);
        DisplayPinCode(
            (p_device),
            (p_pincode),
            methodInvocation);
    }

    if (method_name.compare("RequestPasskey") == 0) {
        Glib::Variant<Glib::DBusObjectPathString> base_device;
        parameters.get_child(base_device, 0);
        Glib::DBusObjectPathString p_device = specialGetter(base_device);

        MethodInvocation methodInvocation(invocation);
        RequestPasskey(
            (p_device),
            methodInvocation);
    }

    if (method_name.compare("DisplayPasskey") == 0) {
        Glib::Variant<Glib::DBusObjectPathString> base_device;
        parameters.get_child(base_device, 0);
        Glib::DBusObjectPathString p_device = specialGetter(base_device);

        Glib::Variant<guint32> base_passkey;
        parameters.get_child(base_passkey, 1);
        guint32 p_passkey = specialGetter(base_passkey);

        Glib::Variant<guint16> base_entered;
        parameters.get_child(base_entered, 2);
        guint16 p_entered = specialGetter(base_entered);

        MethodInvocation methodInvocation(invocation);
        DisplayPasskey(
            (p_device),
            (p_passkey),
            (p_entered),
            methodInvocation);
    }

    if (method_name.compare("RequestConfirmation") == 0) {
        Glib::Variant<Glib::DBusObjectPathString> base_device;
        parameters.get_child(base_device, 0);
        Glib::DBusObjectPathString p_device = specialGetter(base_device);

        Glib::Variant<guint32> base_passkey;
        parameters.get_child(base_passkey, 1);
        guint32 p_passkey = specialGetter(base_passkey);

        MethodInvocation methodInvocation(invocation);
        RequestConfirmation(
            (p_device),
            (p_passkey),
            methodInvocation);
    }

    if (method_name.compare("RequestAuthorization") == 0) {
        Glib::Variant<Glib::DBusObjectPathString> base_device;
        parameters.get_child(base_device, 0);
        Glib::DBusObjectPathString p_device = specialGetter(base_device);

        MethodInvocation methodInvocation(invocation);
        RequestAuthorization(
            (p_device),
            methodInvocation);
    }

    if (method_name.compare("AuthorizeService") == 0) {
        Glib::Variant<Glib::DBusObjectPathString> base_device;
        parameters.get_child(base_device, 0);
        Glib::DBusObjectPathString p_device = specialGetter(base_device);

        Glib::Variant<Glib::ustring> base_uuid;
        parameters.get_child(base_uuid, 1);
        Glib::ustring p_uuid = specialGetter(base_uuid);

        MethodInvocation methodInvocation(invocation);
        AuthorizeService(
            (p_device),
            (p_uuid),
            methodInvocation);
    }

    if (method_name.compare("Cancel") == 0) {
        MethodInvocation methodInvocation(invocation);
        Cancel(
            methodInvocation);
    }

}

void bluez::Agent1Stub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

}

bool bluez::Agent1Stub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    return true;
}


bool bluez::Agent1Stub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
