#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/display_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class DisplayProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<DisplayProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<DisplayProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void ListMonitors(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void ListMonitors_finish (
        std::vector<Glib::ustring> &monitors,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    std::vector<Glib::ustring>
    ListMonitors_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SwitchStyle(
        guint32 style,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SwitchStyle_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SwitchStyle_sync(
        guint32 style,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetDefaultStyle(
        guint32 style,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetDefaultStyle_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetDefaultStyle_sync(
        guint32 style,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void ApplyChanges(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void ApplyChanges_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    ApplyChanges_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void RestoreChanges(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void RestoreChanges_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    RestoreChanges_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetPrimary(
        const Glib::ustring & name,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetPrimary_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetPrimary_sync(
        const Glib::ustring & name,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void Save(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void Save_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    Save_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetWindowScalingFactor(
        gint32 window_scaling_factor,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetWindowScalingFactor_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetWindowScalingFactor_sync(
        gint32 window_scaling_factor,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    guint32 default_style_get(bool *ok = nullptr);
    sigc::signal<void> &default_style_changed() {
        return m_default_style_changed;
    }

    Glib::ustring primary_get(bool *ok = nullptr);
    sigc::signal<void> &primary_changed() {
        return m_primary_changed;
    }

    gint32 window_scaling_factor_get(bool *ok = nullptr);
    sigc::signal<void> &window_scaling_factor_changed() {
        return m_window_scaling_factor_changed;
    }

    sigc::signal<void, bool > MonitorsChanged_signal;

    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    DisplayProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_default_style_changed;
    sigc::signal<void> m_primary_changed;
    sigc::signal<void> m_window_scaling_factor_changed;
};

}// SessionDaemon
}// Kiran

