static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="com.kylinsec.Kiran.SessionDaemon.Display">
        <method name="ListMonitors">
            <arg type="as" name="monitors" direction="out">
                <doc:doc>
                    <doc:summary>The object path list.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>Get the object paths for all the outputs.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="SwitchStyle">
            <arg type="u" name="style" direction="in">
                <doc:doc>
                    <doc:summary>The dispaly style for these connected monitors.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>The display styles contain mirrors(0), extend(1), custom(2) and auto(3). 
                    The mirrors indicates all monitors have the same pos and size;
                    The extend indicates all monitors is extended horizontally;
                    The custom is set from local configuration display.xml;
                    The auto is set by the order of custom, extend, mirrors until success.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="SetDefaultStyle">
            <arg type="u" name="style" direction="in">
                <doc:doc>
                    <doc:summary>The default display style.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>Set the default display style. The display styles contain mirrors(0), extend(1), custom(2) and auto(3).
                    Used when the program starts running or new monitor device is added or existing monitor device is deleted.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="ApplyChanges">
            <doc:doc>
                <doc:description>
                    <doc:para>Apply all changes.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="RestoreChanges">
            <doc:doc>
                <doc:description>
                    <doc:para>Restore all changes.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="SetPrimary">
            <arg type="s" name="name" direction="in">
                <doc:doc>
                    <doc:summary>The name of the monitor.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>Set the primary monitor by name.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="Save">
            <doc:doc>
                <doc:description>
                    <doc:para>Save all changes to file.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <method name="SetWindowScalingFactor">
            <arg type="i" name="window_scaling_factor" direction="in">
                <doc:doc>
                    <doc:summary>The window scaling factor.</doc:summary>
                </doc:doc>
            </arg>
            <doc:doc>
                <doc:description>
                    <doc:para>Set the window scaling factor.</doc:para>
                </doc:description>
            </doc:doc>
        </method>

        <property name="default_style" type="u" access="read">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <doc:doc>
                <doc:description>
                    <doc:para>The default display style. </doc:para>
                </doc:description>
            </doc:doc>
        </property>

        <property name="primary" type="s" access="read">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <doc:doc>
                <doc:description>
                    <doc:para>The name of the primary monitor.</doc:para>
                </doc:description>
            </doc:doc>
        </property>

        <property name="window_scaling_factor" type="i" access="read">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <doc:doc>
                <doc:description>
                    <doc:para>The window scaling factor.</doc:para>
                </doc:description>
            </doc:doc>
        </property>

        <signal name="MonitorsChanged">
            <arg name="placeholder" type="b">
                <doc:doc>
                    <doc:summary>The placeholder.</doc:summary>
                </doc:doc>
            </arg>
        </signal>

    </interface>
</node>
)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/display_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

Kiran::SessionDaemon::DisplayStub::DisplayStub():
    m_interfaceName("com.kylinsec.Kiran.SessionDaemon.Display")
{
    MonitorsChanged_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &DisplayStub::MonitorsChanged_emitter),
            std::vector<Glib::ustring>({""})) );
    MonitorsChanged_selectiveSignal.connect(sigc::mem_fun(this, &DisplayStub::MonitorsChanged_emitter));
}

Kiran::SessionDaemon::DisplayStub::~DisplayStub()
{
    unregister_object();
}

guint Kiran::SessionDaemon::DisplayStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &DisplayStub::on_method_call),
            sigc::mem_fun(this, &DisplayStub::on_interface_get_property),
            sigc::mem_fun(this, &DisplayStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("com.kylinsec.Kiran.SessionDaemon.Display"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void Kiran::SessionDaemon::DisplayStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void Kiran::SessionDaemon::DisplayStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("ListMonitors") == 0) {
        MethodInvocation methodInvocation(invocation);
        ListMonitors(
            methodInvocation);
    }

    if (method_name.compare("SwitchStyle") == 0) {
        Glib::Variant<guint32> base_style;
        parameters.get_child(base_style, 0);
        guint32 p_style = specialGetter(base_style);

        MethodInvocation methodInvocation(invocation);
        SwitchStyle(
            (p_style),
            methodInvocation);
    }

    if (method_name.compare("SetDefaultStyle") == 0) {
        Glib::Variant<guint32> base_style;
        parameters.get_child(base_style, 0);
        guint32 p_style = specialGetter(base_style);

        MethodInvocation methodInvocation(invocation);
        SetDefaultStyle(
            (p_style),
            methodInvocation);
    }

    if (method_name.compare("ApplyChanges") == 0) {
        MethodInvocation methodInvocation(invocation);
        ApplyChanges(
            methodInvocation);
    }

    if (method_name.compare("RestoreChanges") == 0) {
        MethodInvocation methodInvocation(invocation);
        RestoreChanges(
            methodInvocation);
    }

    if (method_name.compare("SetPrimary") == 0) {
        Glib::Variant<Glib::ustring> base_name;
        parameters.get_child(base_name, 0);
        Glib::ustring p_name = specialGetter(base_name);

        MethodInvocation methodInvocation(invocation);
        SetPrimary(
            (p_name),
            methodInvocation);
    }

    if (method_name.compare("Save") == 0) {
        MethodInvocation methodInvocation(invocation);
        Save(
            methodInvocation);
    }

    if (method_name.compare("SetWindowScalingFactor") == 0) {
        Glib::Variant<gint32> base_window_scaling_factor;
        parameters.get_child(base_window_scaling_factor, 0);
        gint32 p_window_scaling_factor = specialGetter(base_window_scaling_factor);

        MethodInvocation methodInvocation(invocation);
        SetWindowScalingFactor(
            (p_window_scaling_factor),
            methodInvocation);
    }

}

void Kiran::SessionDaemon::DisplayStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

    if (property_name.compare("default_style") == 0) {

        property = Glib::Variant<guint32>::create((default_style_get()));
    }

    if (property_name.compare("primary") == 0) {

        property = Glib::Variant<Glib::ustring>::create((primary_get()));
    }

    if (property_name.compare("window_scaling_factor") == 0) {

        property = Glib::Variant<gint32>::create((window_scaling_factor_get()));
    }

}

bool Kiran::SessionDaemon::DisplayStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    return true;
}

void Kiran::SessionDaemon::DisplayStub::MonitorsChanged_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,bool placeholder)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<bool>::create((placeholder)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "com.kylinsec.Kiran.SessionDaemon.Display",
                    "MonitorsChanged",
                    bus_name,
                    params);
        }
    }
}


bool Kiran::SessionDaemon::DisplayStub::default_style_set(guint32 value)
{
    if (default_style_setHandler(value)) {
        Glib::Variant<guint32> value_get =
            Glib::Variant<guint32>::create((default_style_get()));

        emitSignal("default_style", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::DisplayStub::primary_set(const Glib::ustring & value)
{
    if (primary_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((primary_get()));

        emitSignal("primary", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::DisplayStub::window_scaling_factor_set(gint32 value)
{
    if (window_scaling_factor_setHandler(value)) {
        Glib::Variant<gint32> value_get =
            Glib::Variant<gint32>::create((window_scaling_factor_get()));

        emitSignal("window_scaling_factor", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::DisplayStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
