#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/display_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class DisplayStub : public sigc::trackable {
public:
    DisplayStub();
    virtual ~DisplayStub();

    DisplayStub(const DisplayStub &other) = delete;
    DisplayStub(DisplayStub &&other) = delete;
    DisplayStub &operator=(const DisplayStub &other) = delete;
    DisplayStub &operator=(DisplayStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool default_style_set(guint32 value);
    bool primary_set(const Glib::ustring & value);
    bool window_scaling_factor_set(gint32 value);
protected:
    virtual void ListMonitors(
        MethodInvocation &invocation) = 0;
    virtual void SwitchStyle(
        guint32 style,
        MethodInvocation &invocation) = 0;
    virtual void SetDefaultStyle(
        guint32 style,
        MethodInvocation &invocation) = 0;
    virtual void ApplyChanges(
        MethodInvocation &invocation) = 0;
    virtual void RestoreChanges(
        MethodInvocation &invocation) = 0;
    virtual void SetPrimary(
        const Glib::ustring & name,
        MethodInvocation &invocation) = 0;
    virtual void Save(
        MethodInvocation &invocation) = 0;
    virtual void SetWindowScalingFactor(
        gint32 window_scaling_factor,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool default_style_setHandler(guint32 value) = 0;
    virtual guint32 default_style_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool primary_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring primary_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool window_scaling_factor_setHandler(gint32 value) = 0;
    virtual gint32 window_scaling_factor_get() = 0;

    void MonitorsChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, bool);
    sigc::signal<void, bool> MonitorsChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, bool> MonitorsChanged_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class DisplayStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret(const std::vector<Glib::ustring> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::vector<Glib::ustring>> var0 =
            Glib::Variant<std::vector<Glib::ustring>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionDaemon
} // Kiran

