#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/greeter_dbus_common.h"

namespace Kiran {
namespace SystemDaemon {

class GreeterProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<GreeterProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<GreeterProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void SetBackground(
        const Glib::ustring & file_path,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetBackground_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetBackground_sync(
        const Glib::ustring & file_path,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAutologinUser(
        const Glib::ustring & user_name,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAutologinUser_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetAutologinUser_sync(
        const Glib::ustring & user_name,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAutologinTimeout(
        guint64 seconds,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAutologinTimeout_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetAutologinTimeout_sync(
        guint64 seconds,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetHideUserList(
        bool hide,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetHideUserList_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetHideUserList_sync(
        bool hide,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAllowManualLogin(
        bool allow,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAllowManualLogin_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetAllowManualLogin_sync(
        bool allow,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetScaleMode(
        guint16 mode,
        guint16 factor,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetScaleMode_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetScaleMode_sync(
        guint16 mode,        guint16 factor,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    Glib::ustring background_get(bool *ok = nullptr);
    sigc::signal<void> &background_changed() {
        return m_background_changed;
    }

    Glib::ustring autologin_user_get(bool *ok = nullptr);
    sigc::signal<void> &autologin_user_changed() {
        return m_autologin_user_changed;
    }

    guint64 autologin_timeout_get(bool *ok = nullptr);
    sigc::signal<void> &autologin_timeout_changed() {
        return m_autologin_timeout_changed;
    }

    bool allow_manual_login_get(bool *ok = nullptr);
    sigc::signal<void> &allow_manual_login_changed() {
        return m_allow_manual_login_changed;
    }

    bool hide_user_list_get(bool *ok = nullptr);
    sigc::signal<void> &hide_user_list_changed() {
        return m_hide_user_list_changed;
    }

    guint16 scale_mode_get(bool *ok = nullptr);
    sigc::signal<void> &scale_mode_changed() {
        return m_scale_mode_changed;
    }

    guint16 scale_factor_get(bool *ok = nullptr);
    sigc::signal<void> &scale_factor_changed() {
        return m_scale_factor_changed;
    }


    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    GreeterProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_background_changed;
    sigc::signal<void> m_autologin_user_changed;
    sigc::signal<void> m_autologin_timeout_changed;
    sigc::signal<void> m_allow_manual_login_changed;
    sigc::signal<void> m_hide_user_list_changed;
    sigc::signal<void> m_scale_mode_changed;
    sigc::signal<void> m_scale_factor_changed;
};

}// SystemDaemon
}// Kiran

