#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/greeter_dbus_common.h"

namespace Kiran {
namespace SystemDaemon {

class GreeterStub : public sigc::trackable {
public:
    GreeterStub();
    virtual ~GreeterStub();

    GreeterStub(const GreeterStub &other) = delete;
    GreeterStub(GreeterStub &&other) = delete;
    GreeterStub &operator=(const GreeterStub &other) = delete;
    GreeterStub &operator=(GreeterStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool background_set(const Glib::ustring & value);
    bool autologin_user_set(const Glib::ustring & value);
    bool autologin_timeout_set(guint64 value);
    bool allow_manual_login_set(bool value);
    bool hide_user_list_set(bool value);
    bool scale_mode_set(guint16 value);
    bool scale_factor_set(guint16 value);
protected:
    virtual void SetBackground(
        const Glib::ustring & file_path,
        MethodInvocation &invocation) = 0;
    virtual void SetAutologinUser(
        const Glib::ustring & user_name,
        MethodInvocation &invocation) = 0;
    virtual void SetAutologinTimeout(
        guint64 seconds,
        MethodInvocation &invocation) = 0;
    virtual void SetHideUserList(
        bool hide,
        MethodInvocation &invocation) = 0;
    virtual void SetAllowManualLogin(
        bool allow,
        MethodInvocation &invocation) = 0;
    virtual void SetScaleMode(
        guint16 mode,
        guint16 factor,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool background_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring background_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool autologin_user_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring autologin_user_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool autologin_timeout_setHandler(guint64 value) = 0;
    virtual guint64 autologin_timeout_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool allow_manual_login_setHandler(bool value) = 0;
    virtual bool allow_manual_login_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool hide_user_list_setHandler(bool value) = 0;
    virtual bool hide_user_list_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool scale_mode_setHandler(guint16 value) = 0;
    virtual guint16 scale_mode_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool scale_factor_setHandler(guint16 value) = 0;
    virtual guint16 scale_factor_get() = 0;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class GreeterStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SystemDaemon
} // Kiran

