#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace Kiran {
namespace SessionDaemon {

class KeybindingTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase AddCustomShortcut_pack(
        const Glib::ustring & arg_name,
        const Glib::ustring & arg_action,
        const Glib::ustring & arg_key_combination) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> name_param =
            Glib::Variant<Glib::ustring>::create(arg_name);
        params.push_back(name_param);

        Glib::Variant<Glib::ustring> action_param =
            Glib::Variant<Glib::ustring>::create(arg_action);
        params.push_back(action_param);

        Glib::Variant<Glib::ustring> key_combination_param =
            Glib::Variant<Glib::ustring>::create(arg_key_combination);
        params.push_back(key_combination_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase ModifyCustomShortcut_pack(
        const Glib::ustring & arg_uid,
        const Glib::ustring & arg_name,
        const Glib::ustring & arg_action,
        const Glib::ustring & arg_key_combination) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> uid_param =
            Glib::Variant<Glib::ustring>::create(arg_uid);
        params.push_back(uid_param);

        Glib::Variant<Glib::ustring> name_param =
            Glib::Variant<Glib::ustring>::create(arg_name);
        params.push_back(name_param);

        Glib::Variant<Glib::ustring> action_param =
            Glib::Variant<Glib::ustring>::create(arg_action);
        params.push_back(action_param);

        Glib::Variant<Glib::ustring> key_combination_param =
            Glib::Variant<Glib::ustring>::create(arg_key_combination);
        params.push_back(key_combination_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase DeleteCustomShortcut_pack(
        const Glib::ustring & arg_uid) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_uid);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetCustomShortcut_pack(
        const Glib::ustring & arg_uid) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_uid);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase ModifySystemShortcut_pack(
        const Glib::ustring & arg_uid,
        const Glib::ustring & arg_key_combination) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> uid_param =
            Glib::Variant<Glib::ustring>::create(arg_uid);
        params.push_back(uid_param);

        Glib::Variant<Glib::ustring> key_combination_param =
            Glib::Variant<Glib::ustring>::create(arg_key_combination);
        params.push_back(key_combination_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetSystemShortcut_pack(
        const Glib::ustring & arg_uid) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_uid);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // SessionDaemon
} // Kiran


