#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/keybinding_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class KeybindingStub : public sigc::trackable {
public:
    KeybindingStub();
    virtual ~KeybindingStub();

    KeybindingStub(const KeybindingStub &other) = delete;
    KeybindingStub(KeybindingStub &&other) = delete;
    KeybindingStub &operator=(const KeybindingStub &other) = delete;
    KeybindingStub &operator=(KeybindingStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

protected:
    virtual void AddCustomShortcut(
        const Glib::ustring & name,
        const Glib::ustring & action,
        const Glib::ustring & key_combination,
        MethodInvocation &invocation) = 0;
    virtual void ModifyCustomShortcut(
        const Glib::ustring & uid,
        const Glib::ustring & name,
        const Glib::ustring & action,
        const Glib::ustring & key_combination,
        MethodInvocation &invocation) = 0;
    virtual void DeleteCustomShortcut(
        const Glib::ustring & uid,
        MethodInvocation &invocation) = 0;
    virtual void GetCustomShortcut(
        const Glib::ustring & uid,
        MethodInvocation &invocation) = 0;
    virtual void ListCustomShortcuts(
        MethodInvocation &invocation) = 0;
    virtual void ModifySystemShortcut(
        const Glib::ustring & uid,
        const Glib::ustring & key_combination,
        MethodInvocation &invocation) = 0;
    virtual void GetSystemShortcut(
        const Glib::ustring & uid,
        MethodInvocation &invocation) = 0;
    virtual void ListSystemShortcuts(
        MethodInvocation &invocation) = 0;
    virtual void ListShortcuts(
        MethodInvocation &invocation) = 0;

    void Added_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::ustring &);
    sigc::signal<void, const Glib::ustring &> Added_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::ustring &> Added_selectiveSignal;

    void Deleted_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::ustring &);
    sigc::signal<void, const Glib::ustring &> Deleted_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::ustring &> Deleted_selectiveSignal;

    void Changed_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::ustring &);
    sigc::signal<void, const Glib::ustring &> Changed_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::ustring &> Changed_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class KeybindingStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret(const Glib::ustring & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<Glib::ustring> var0 =
            Glib::Variant<Glib::ustring>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionDaemon
} // Kiran

