#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/keyboard_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class KeyboardStub : public sigc::trackable {
public:
    KeyboardStub();
    virtual ~KeyboardStub();

    KeyboardStub(const KeyboardStub &other) = delete;
    KeyboardStub(KeyboardStub &&other) = delete;
    KeyboardStub &operator=(const KeyboardStub &other) = delete;
    KeyboardStub &operator=(KeyboardStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool repeat_enabled_set(bool value);
    bool repeat_delay_set(gint32 value);
    bool repeat_interval_set(gint32 value);
    bool layouts_set(const std::vector<Glib::ustring> & value);
    bool options_set(const std::vector<Glib::ustring> & value);
protected:
    virtual void AddLayout(
        const Glib::ustring & layout,
        MethodInvocation &invocation) = 0;
    virtual void DelLayout(
        const Glib::ustring & layout,
        MethodInvocation &invocation) = 0;
    virtual void ApplyLayout(
        const Glib::ustring & layout,
        MethodInvocation &invocation) = 0;
    virtual void GetValidLayouts(
        MethodInvocation &invocation) = 0;
    virtual void AddLayoutOption(
        const Glib::ustring & option,
        MethodInvocation &invocation) = 0;
    virtual void DelLayoutOption(
        const Glib::ustring & option,
        MethodInvocation &invocation) = 0;
    virtual void ClearLayoutOption(
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool repeat_enabled_setHandler(bool value) = 0;
    virtual bool repeat_enabled_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool repeat_delay_setHandler(gint32 value) = 0;
    virtual gint32 repeat_delay_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool repeat_interval_setHandler(gint32 value) = 0;
    virtual gint32 repeat_interval_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool layouts_setHandler(const std::vector<Glib::ustring> & value) = 0;
    virtual std::vector<Glib::ustring> layouts_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool options_setHandler(const std::vector<Glib::ustring> & value) = 0;
    virtual std::vector<Glib::ustring> options_get() = 0;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class KeyboardStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const Glib::ustring & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<Glib::ustring> var0 =
            Glib::Variant<Glib::ustring>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionDaemon
} // Kiran

