#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace Kiran {
namespace SessionDaemon {
namespace Display {

class MonitorTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase Enable_pack(
        bool arg_enabled) {
        Glib::VariantContainerBase base;
        Glib::Variant<bool> params =
            Glib::Variant<bool>::create(arg_enabled);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetMode_pack(
        guint32 arg_width,
        guint32 arg_height,
        double arg_refresh_rate) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<guint32> width_param =
            Glib::Variant<guint32>::create(arg_width);
        params.push_back(width_param);

        Glib::Variant<guint32> height_param =
            Glib::Variant<guint32>::create(arg_height);
        params.push_back(height_param);

        Glib::Variant<double> refresh_rate_param =
            Glib::Variant<double>::create(arg_refresh_rate);
        params.push_back(refresh_rate_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetModeById_pack(
        guint32 arg_id) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint32> params =
            Glib::Variant<guint32>::create(arg_id);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetModeBySize_pack(
        guint32 arg_width,
        guint32 arg_height) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<guint32> width_param =
            Glib::Variant<guint32>::create(arg_width);
        params.push_back(width_param);

        Glib::Variant<guint32> height_param =
            Glib::Variant<guint32>::create(arg_height);
        params.push_back(height_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetPosition_pack(
        gint32 arg_x,
        gint32 arg_y) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint32> x_param =
            Glib::Variant<gint32>::create(arg_x);
        params.push_back(x_param);

        Glib::Variant<gint32> y_param =
            Glib::Variant<gint32>::create(arg_y);
        params.push_back(y_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetRotation_pack(
        guint16 arg_rotation) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint16> params =
            Glib::Variant<guint16>::create(arg_rotation);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetReflect_pack(
        guint16 arg_reflect) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint16> params =
            Glib::Variant<guint16>::create(arg_reflect);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // Display
} // SessionDaemon
} // Kiran


