#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/monitor_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {
namespace Display {

class MonitorStub : public sigc::trackable {
public:
    MonitorStub();
    virtual ~MonitorStub();

    MonitorStub(const MonitorStub &other) = delete;
    MonitorStub(MonitorStub &&other) = delete;
    MonitorStub &operator=(const MonitorStub &other) = delete;
    MonitorStub &operator=(MonitorStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool id_set(guint32 value);
    bool name_set(const Glib::ustring & value);
    bool connected_set(bool value);
    bool enabled_set(bool value);
    bool x_set(gint32 value);
    bool y_set(gint32 value);
    bool rotation_set(guint16 value);
    bool reflect_set(guint16 value);
    bool rotations_set(const std::vector<guint16> & value);
    bool reflects_set(const std::vector<guint16> & value);
    bool current_mode_set(guint32 value);
    bool modes_set(const std::vector<guint32> & value);
    bool npreferred_set(gint32 value);
protected:
    virtual void Enable(
        bool enabled,
        MethodInvocation &invocation) = 0;
    virtual void ListModes(
        MethodInvocation &invocation) = 0;
    virtual void ListPreferredModes(
        MethodInvocation &invocation) = 0;
    virtual void GetCurrentMode(
        MethodInvocation &invocation) = 0;
    virtual void SetMode(
        guint32 width,
        guint32 height,
        double refresh_rate,
        MethodInvocation &invocation) = 0;
    virtual void SetModeById(
        guint32 id,
        MethodInvocation &invocation) = 0;
    virtual void SetModeBySize(
        guint32 width,
        guint32 height,
        MethodInvocation &invocation) = 0;
    virtual void SetPosition(
        gint32 x,
        gint32 y,
        MethodInvocation &invocation) = 0;
    virtual void SetRotation(
        guint16 rotation,
        MethodInvocation &invocation) = 0;
    virtual void SetReflect(
        guint16 reflect,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool id_setHandler(guint32 value) = 0;
    virtual guint32 id_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool name_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring name_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool connected_setHandler(bool value) = 0;
    virtual bool connected_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool enabled_setHandler(bool value) = 0;
    virtual bool enabled_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool x_setHandler(gint32 value) = 0;
    virtual gint32 x_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool y_setHandler(gint32 value) = 0;
    virtual gint32 y_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool rotation_setHandler(guint16 value) = 0;
    virtual guint16 rotation_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool reflect_setHandler(guint16 value) = 0;
    virtual guint16 reflect_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool rotations_setHandler(const std::vector<guint16> & value) = 0;
    virtual std::vector<guint16> rotations_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool reflects_setHandler(const std::vector<guint16> & value) = 0;
    virtual std::vector<guint16> reflects_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool current_mode_setHandler(guint32 value) = 0;
    virtual guint32 current_mode_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool modes_setHandler(const std::vector<guint32> & value) = 0;
    virtual std::vector<guint32> modes_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool npreferred_setHandler(gint32 value) = 0;
    virtual gint32 npreferred_get() = 0;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class MonitorStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const std::vector<std::tuple<guint32,guint32,guint32,double>> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::vector<std::tuple<guint32,guint32,guint32,double>>> var0 =
            Glib::Variant<std::vector<std::tuple<guint32,guint32,guint32,double>>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const std::tuple<guint32,guint32,guint32,double> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::tuple<guint32,guint32,guint32,double>> var0 =
            Glib::Variant<std::tuple<guint32,guint32,guint32,double>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // Display
} // SessionDaemon
} // Kiran

