#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/mouse_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class MouseProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<MouseProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<MouseProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void Reset(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void Reset_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    Reset_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    bool left_handed_get(bool *ok = nullptr);
    void left_handed_set(bool, const Gio::SlotAsyncReady &);
    void left_handed_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void left_handed_set_sync(bool);
    sigc::signal<void> &left_handed_changed() {
        return m_left_handed_changed;
    }

    double motion_acceleration_get(bool *ok = nullptr);
    void motion_acceleration_set(double, const Gio::SlotAsyncReady &);
    void motion_acceleration_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void motion_acceleration_set_sync(double);
    sigc::signal<void> &motion_acceleration_changed() {
        return m_motion_acceleration_changed;
    }

    bool middle_emulation_enabled_get(bool *ok = nullptr);
    void middle_emulation_enabled_set(bool, const Gio::SlotAsyncReady &);
    void middle_emulation_enabled_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void middle_emulation_enabled_set_sync(bool);
    sigc::signal<void> &middle_emulation_enabled_changed() {
        return m_middle_emulation_enabled_changed;
    }

    bool natural_scroll_get(bool *ok = nullptr);
    void natural_scroll_set(bool, const Gio::SlotAsyncReady &);
    void natural_scroll_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void natural_scroll_set_sync(bool);
    sigc::signal<void> &natural_scroll_changed() {
        return m_natural_scroll_changed;
    }


    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    MouseProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_left_handed_changed;
    sigc::signal<void> m_motion_acceleration_changed;
    sigc::signal<void> m_middle_emulation_enabled_changed;
    sigc::signal<void> m_natural_scroll_changed;
};

}// SessionDaemon
}// Kiran

