#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/network_proxy_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {
namespace Network {

class ProxyProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<ProxyProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<ProxyProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void SetMode(
        gint32 mode,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetMode_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetMode_sync(
        gint32 mode,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetManualProxy(
        const Glib::ustring & options,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetManualProxy_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetManualProxy_sync(
        const Glib::ustring & options,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetManualProxy(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetManualProxy_finish (
        Glib::ustring &options,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    Glib::ustring
    GetManualProxy_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAutoProxy(
        const Glib::ustring & url,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetAutoProxy_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetAutoProxy_sync(
        const Glib::ustring & url,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    gint32 mode_get(bool *ok = nullptr);
    sigc::signal<void> &mode_changed() {
        return m_mode_changed;
    }

    Glib::ustring proxy_url_get(bool *ok = nullptr);
    sigc::signal<void> &proxy_url_changed() {
        return m_proxy_url_changed;
    }

    sigc::signal<void, guchar > ManualProxyChanged_signal;

    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    ProxyProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_mode_changed;
    sigc::signal<void> m_proxy_url_changed;
};

}// Network
}// SessionDaemon
}// Kiran

