static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="com.kylinsec.Kiran.SessionDaemon.Network.Proxy">

        <method name="SetMode">
            <arg name="mode" direction="in" type="i">
                <description>The proxy mode. Refer to NetworkProxyMode in network-i.h</description>
            </arg>
            <description>Set the proxy mode.</description>
        </method>

        <method name="SetManualProxy">
            <arg name="options" direction="in" type="s">
                <description>The proxy options which is a json string.</description>
            </arg>
            <description>Set manual proxy arguments.</description>
        </method>

        <method name="GetManualProxy">
            <arg name="options" direction="out" type="s">
                <description>The proxy options which is a json string.</description>
            </arg>
            <description>Get manual proxy arguments.</description>
        </method>

        <method name="SetAutoProxy">
            <arg name="url" direction="in" type="s">
                <description>The proxy url.</description>
            </arg>
            <description>Set the proxy url .</description>
        </method>

        <property name="mode" type="i" access="read">
            <description>The proxy mode.</description>
        </property>

        <property name="proxy_url" type="s" access="read">
            <description>URL that provides proxy configuration values. When mode is auto, this URL is used to look up proxy information for all protocols.</description>
        </property>

        <signal name="ManualProxyChanged">
            <arg name="placeholder" type="y" />
            <description>The manual proxy arguments is changed.</description>
        </signal>

    </interface>
</node>)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/network_proxy_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

Kiran::SessionDaemon::Network::ProxyStub::ProxyStub():
    m_interfaceName("com.kylinsec.Kiran.SessionDaemon.Network.Proxy")
{
    ManualProxyChanged_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &ProxyStub::ManualProxyChanged_emitter),
            std::vector<Glib::ustring>({""})) );
    ManualProxyChanged_selectiveSignal.connect(sigc::mem_fun(this, &ProxyStub::ManualProxyChanged_emitter));
}

Kiran::SessionDaemon::Network::ProxyStub::~ProxyStub()
{
    unregister_object();
}

guint Kiran::SessionDaemon::Network::ProxyStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &ProxyStub::on_method_call),
            sigc::mem_fun(this, &ProxyStub::on_interface_get_property),
            sigc::mem_fun(this, &ProxyStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("com.kylinsec.Kiran.SessionDaemon.Network.Proxy"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void Kiran::SessionDaemon::Network::ProxyStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void Kiran::SessionDaemon::Network::ProxyStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("SetMode") == 0) {
        Glib::Variant<gint32> base_mode;
        parameters.get_child(base_mode, 0);
        gint32 p_mode = specialGetter(base_mode);

        MethodInvocation methodInvocation(invocation);
        SetMode(
            (p_mode),
            methodInvocation);
    }

    if (method_name.compare("SetManualProxy") == 0) {
        Glib::Variant<Glib::ustring> base_options;
        parameters.get_child(base_options, 0);
        Glib::ustring p_options = specialGetter(base_options);

        MethodInvocation methodInvocation(invocation);
        SetManualProxy(
            (p_options),
            methodInvocation);
    }

    if (method_name.compare("GetManualProxy") == 0) {
        MethodInvocation methodInvocation(invocation);
        GetManualProxy(
            methodInvocation);
    }

    if (method_name.compare("SetAutoProxy") == 0) {
        Glib::Variant<Glib::ustring> base_url;
        parameters.get_child(base_url, 0);
        Glib::ustring p_url = specialGetter(base_url);

        MethodInvocation methodInvocation(invocation);
        SetAutoProxy(
            (p_url),
            methodInvocation);
    }

}

void Kiran::SessionDaemon::Network::ProxyStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

    if (property_name.compare("mode") == 0) {

        property = Glib::Variant<gint32>::create((mode_get()));
    }

    if (property_name.compare("proxy_url") == 0) {

        property = Glib::Variant<Glib::ustring>::create((proxy_url_get()));
    }

}

bool Kiran::SessionDaemon::Network::ProxyStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    return true;
}

void Kiran::SessionDaemon::Network::ProxyStub::ManualProxyChanged_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,guchar placeholder)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<guchar>::create((placeholder)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "com.kylinsec.Kiran.SessionDaemon.Network.Proxy",
                    "ManualProxyChanged",
                    bus_name,
                    params);
        }
    }
}


bool Kiran::SessionDaemon::Network::ProxyStub::mode_set(gint32 value)
{
    if (mode_setHandler(value)) {
        Glib::Variant<gint32> value_get =
            Glib::Variant<gint32>::create((mode_get()));

        emitSignal("mode", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::Network::ProxyStub::proxy_url_set(const Glib::ustring & value)
{
    if (proxy_url_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((proxy_url_get()));

        emitSignal("proxy_url", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::Network::ProxyStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
