#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/object_manager_dbus_common.h"

namespace DBus {

class ObjectManagerStub : public sigc::trackable {
public:
    ObjectManagerStub();
    virtual ~ObjectManagerStub();

    ObjectManagerStub(const ObjectManagerStub &other) = delete;
    ObjectManagerStub(ObjectManagerStub &&other) = delete;
    ObjectManagerStub &operator=(const ObjectManagerStub &other) = delete;
    ObjectManagerStub &operator=(ObjectManagerStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

protected:
    virtual void GetManagedObjects(
        MethodInvocation &invocation) = 0;

    void InterfacesAdded_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &, const std::map<Glib::ustring,std::map<Glib::ustring,Glib::VariantBase>> &);
    sigc::signal<void, const Glib::DBusObjectPathString &, const std::map<Glib::ustring,std::map<Glib::ustring,Glib::VariantBase>> &> InterfacesAdded_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &, const std::map<Glib::ustring,std::map<Glib::ustring,Glib::VariantBase>> &> InterfacesAdded_selectiveSignal;

    void InterfacesRemoved_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::DBusObjectPathString &, const std::vector<Glib::ustring> &);
    sigc::signal<void, const Glib::DBusObjectPathString &, const std::vector<Glib::ustring> &> InterfacesRemoved_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::DBusObjectPathString &, const std::vector<Glib::ustring> &> InterfacesRemoved_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class ObjectManagerStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret(const std::map<Glib::DBusObjectPathString,std::map<Glib::ustring,std::map<Glib::ustring,Glib::VariantBase>>> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::map<Glib::DBusObjectPathString,std::map<Glib::ustring,std::map<Glib::ustring,Glib::VariantBase>>>> var0 =
            Glib::Variant<std::map<Glib::DBusObjectPathString,std::map<Glib::ustring,std::map<Glib::ustring,Glib::VariantBase>>>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // DBus

