#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace Kiran {
namespace SessionDaemon {

class PowerTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase SetIdleAction_pack(
        gint32 arg_device,
        gint32 arg_supply,
        gint32 arg_idle_timeout,
        gint32 arg_action) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint32> device_param =
            Glib::Variant<gint32>::create(arg_device);
        params.push_back(device_param);

        Glib::Variant<gint32> supply_param =
            Glib::Variant<gint32>::create(arg_supply);
        params.push_back(supply_param);

        Glib::Variant<gint32> idle_timeout_param =
            Glib::Variant<gint32>::create(arg_idle_timeout);
        params.push_back(idle_timeout_param);

        Glib::Variant<gint32> action_param =
            Glib::Variant<gint32>::create(arg_action);
        params.push_back(action_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetIdleAction_pack(
        gint32 arg_device,
        gint32 arg_supply) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint32> device_param =
            Glib::Variant<gint32>::create(arg_device);
        params.push_back(device_param);

        Glib::Variant<gint32> supply_param =
            Glib::Variant<gint32>::create(arg_supply);
        params.push_back(supply_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetEventAction_pack(
        gint32 arg_event,
        gint32 arg_action) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint32> event_param =
            Glib::Variant<gint32>::create(arg_event);
        params.push_back(event_param);

        Glib::Variant<gint32> action_param =
            Glib::Variant<gint32>::create(arg_action);
        params.push_back(action_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetEventAction_pack(
        gint32 arg_event) {
        Glib::VariantContainerBase base;
        Glib::Variant<gint32> params =
            Glib::Variant<gint32>::create(arg_event);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetBrightness_pack(
        gint32 arg_device,
        gint32 arg_brightness_percentage) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint32> device_param =
            Glib::Variant<gint32>::create(arg_device);
        params.push_back(device_param);

        Glib::Variant<gint32> brightness_percentage_param =
            Glib::Variant<gint32>::create(arg_brightness_percentage);
        params.push_back(brightness_percentage_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetBrightness_pack(
        gint32 arg_device) {
        Glib::VariantContainerBase base;
        Glib::Variant<gint32> params =
            Glib::Variant<gint32>::create(arg_device);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetIdleDimmed_pack(
        gint32 arg_scale) {
        Glib::VariantContainerBase base;
        Glib::Variant<gint32> params =
            Glib::Variant<gint32>::create(arg_scale);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // SessionDaemon
} // Kiran


