#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/power_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class PowerProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<PowerProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<PowerProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void SetIdleAction(
        gint32 device,
        gint32 supply,
        gint32 idle_timeout,
        gint32 action,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetIdleAction_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetIdleAction_sync(
        gint32 device,        gint32 supply,        gint32 idle_timeout,        gint32 action,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetIdleAction(
        gint32 device,
        gint32 supply,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetIdleAction_finish (
        std::tuple<gint32,gint32> &idle_action,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    std::tuple<gint32,gint32>
    GetIdleAction_sync(
        gint32 device,        gint32 supply,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetEventAction(
        gint32 event,
        gint32 action,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetEventAction_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetEventAction_sync(
        gint32 event,        gint32 action,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetEventAction(
        gint32 event,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetEventAction_finish (
        gint32 &action,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    gint32
    GetEventAction_sync(
        gint32 event,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetBrightness(
        gint32 device,
        gint32 brightness_percentage,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetBrightness_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetBrightness_sync(
        gint32 device,        gint32 brightness_percentage,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetBrightness(
        gint32 device,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void GetBrightness_finish (
        gint32 &brightness_percentage,
        const Glib::RefPtr<Gio::AsyncResult> &res);

    gint32
    GetBrightness_sync(
        gint32 device,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetIdleDimmed(
        gint32 scale,
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void SetIdleDimmed_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    SetIdleDimmed_sync(
        gint32 scale,const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    bool on_battery_get(bool *ok = nullptr);
    sigc::signal<void> &on_battery_changed() {
        return m_on_battery_changed;
    }

    bool lid_is_present_get(bool *ok = nullptr);
    sigc::signal<void> &lid_is_present_changed() {
        return m_lid_is_present_changed;
    }

    gint32 idle_dimmed_scale_get(bool *ok = nullptr);
    sigc::signal<void> &idle_dimmed_scale_changed() {
        return m_idle_dimmed_scale_changed;
    }

    sigc::signal<void, gint32, gint32 > IdleActionChanged_signal;
    sigc::signal<void, gint32 > EventActionChanged_signal;
    sigc::signal<void, gint32 > BrightnessChanged_signal;

    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    PowerProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_on_battery_changed;
    sigc::signal<void> m_lid_is_present_changed;
    sigc::signal<void> m_idle_dimmed_scale_changed;
};

}// SessionDaemon
}// Kiran

