#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/power_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class PowerStub : public sigc::trackable {
public:
    PowerStub();
    virtual ~PowerStub();

    PowerStub(const PowerStub &other) = delete;
    PowerStub(PowerStub &&other) = delete;
    PowerStub &operator=(const PowerStub &other) = delete;
    PowerStub &operator=(PowerStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool on_battery_set(bool value);
    bool lid_is_present_set(bool value);
    bool idle_dimmed_scale_set(gint32 value);
protected:
    virtual void SetIdleAction(
        gint32 device,
        gint32 supply,
        gint32 idle_timeout,
        gint32 action,
        MethodInvocation &invocation) = 0;
    virtual void GetIdleAction(
        gint32 device,
        gint32 supply,
        MethodInvocation &invocation) = 0;
    virtual void SetEventAction(
        gint32 event,
        gint32 action,
        MethodInvocation &invocation) = 0;
    virtual void GetEventAction(
        gint32 event,
        MethodInvocation &invocation) = 0;
    virtual void SetBrightness(
        gint32 device,
        gint32 brightness_percentage,
        MethodInvocation &invocation) = 0;
    virtual void GetBrightness(
        gint32 device,
        MethodInvocation &invocation) = 0;
    virtual void SetIdleDimmed(
        gint32 scale,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool on_battery_setHandler(bool value) = 0;
    virtual bool on_battery_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool lid_is_present_setHandler(bool value) = 0;
    virtual bool lid_is_present_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool idle_dimmed_scale_setHandler(gint32 value) = 0;
    virtual gint32 idle_dimmed_scale_get() = 0;

    void IdleActionChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, gint32, gint32);
    sigc::signal<void, gint32, gint32> IdleActionChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, gint32, gint32> IdleActionChanged_selectiveSignal;

    void EventActionChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, gint32);
    sigc::signal<void, gint32> EventActionChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, gint32> EventActionChanged_selectiveSignal;

    void BrightnessChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, gint32);
    sigc::signal<void, gint32> BrightnessChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, gint32> BrightnessChanged_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class PowerStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const std::tuple<gint32,gint32> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::tuple<gint32,gint32>> var0 =
            Glib::Variant<std::tuple<gint32,gint32>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(gint32 p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<gint32> var0 =
            Glib::Variant<gint32>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionDaemon
} // Kiran

