#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace Kiran {
namespace SystemDaemon {

class TimeDateTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase SetTime_pack(
        gint64 arg_requested_time,
        bool arg_relative) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint64> requested_time_param =
            Glib::Variant<gint64>::create(arg_requested_time);
        params.push_back(requested_time_param);

        Glib::Variant<bool> relative_param =
            Glib::Variant<bool>::create(arg_relative);
        params.push_back(relative_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetTimezone_pack(
        const Glib::ustring & arg_time_zone) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_time_zone);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetLocalRTC_pack(
        bool arg_local,
        bool arg_adjust_system) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<bool> local_param =
            Glib::Variant<bool>::create(arg_local);
        params.push_back(local_param);

        Glib::Variant<bool> adjust_system_param =
            Glib::Variant<bool>::create(arg_adjust_system);
        params.push_back(adjust_system_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetNTP_pack(
        bool arg_active) {
        Glib::VariantContainerBase base;
        Glib::Variant<bool> params =
            Glib::Variant<bool>::create(arg_active);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetDateFormatList_pack(
        gint32 arg_type) {
        Glib::VariantContainerBase base;
        Glib::Variant<gint32> params =
            Glib::Variant<gint32>::create(arg_type);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetDateFormatByIndex_pack(
        gint32 arg_type,
        gint32 arg_index) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<gint32> type_param =
            Glib::Variant<gint32>::create(arg_type);
        params.push_back(type_param);

        Glib::Variant<gint32> index_param =
            Glib::Variant<gint32>::create(arg_index);
        params.push_back(index_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetHourFormat_pack(
        gint32 arg_format) {
        Glib::VariantContainerBase base;
        Glib::Variant<gint32> params =
            Glib::Variant<gint32>::create(arg_format);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase EnableSecondsShowing_pack(
        bool arg_enabled) {
        Glib::VariantContainerBase base;
        Glib::Variant<bool> params =
            Glib::Variant<bool>::create(arg_enabled);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // SystemDaemon
} // Kiran


