#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/timedate_dbus_common.h"

namespace Kiran {
namespace SystemDaemon {

class TimeDateStub : public sigc::trackable {
public:
    TimeDateStub();
    virtual ~TimeDateStub();

    TimeDateStub(const TimeDateStub &other) = delete;
    TimeDateStub(TimeDateStub &&other) = delete;
    TimeDateStub &operator=(const TimeDateStub &other) = delete;
    TimeDateStub &operator=(TimeDateStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool time_zone_set(const Glib::ustring & value);
    bool local_rtc_set(bool value);
    bool can_ntp_set(bool value);
    bool ntp_set(bool value);
    bool system_time_set(guint64 value);
    bool rtc_time_set(guint64 value);
    bool date_long_format_index_set(gint32 value);
    bool date_short_format_index_set(gint32 value);
    bool hour_format_set(gint32 value);
    bool seconds_showing_set(bool value);
protected:
    virtual void SetTime(
        gint64 requested_time,
        bool relative,
        MethodInvocation &invocation) = 0;
    virtual void SetTimezone(
        const Glib::ustring & time_zone,
        MethodInvocation &invocation) = 0;
    virtual void GetZoneList(
        MethodInvocation &invocation) = 0;
    virtual void SetLocalRTC(
        bool local,
        bool adjust_system,
        MethodInvocation &invocation) = 0;
    virtual void SetNTP(
        bool active,
        MethodInvocation &invocation) = 0;
    virtual void GetDateFormatList(
        gint32 type,
        MethodInvocation &invocation) = 0;
    virtual void SetDateFormatByIndex(
        gint32 type,
        gint32 index,
        MethodInvocation &invocation) = 0;
    virtual void SetHourFormat(
        gint32 format,
        MethodInvocation &invocation) = 0;
    virtual void EnableSecondsShowing(
        bool enabled,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool time_zone_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring time_zone_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool local_rtc_setHandler(bool value) = 0;
    virtual bool local_rtc_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool can_ntp_setHandler(bool value) = 0;
    virtual bool can_ntp_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool ntp_setHandler(bool value) = 0;
    virtual bool ntp_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool system_time_setHandler(guint64 value) = 0;
    virtual guint64 system_time_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool rtc_time_setHandler(guint64 value) = 0;
    virtual guint64 rtc_time_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool date_long_format_index_setHandler(gint32 value) = 0;
    virtual gint32 date_long_format_index_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool date_short_format_index_setHandler(gint32 value) = 0;
    virtual gint32 date_short_format_index_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool hour_format_setHandler(gint32 value) = 0;
    virtual gint32 hour_format_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool seconds_showing_setHandler(bool value) = 0;
    virtual bool seconds_showing_get() = 0;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class TimeDateStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const std::vector<std::tuple<Glib::ustring,Glib::ustring,gint64>> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::vector<std::tuple<Glib::ustring,Glib::ustring,gint64>>> var0 =
            Glib::Variant<std::vector<std::tuple<Glib::ustring,Glib::ustring,gint64>>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const std::vector<Glib::ustring> & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<std::vector<Glib::ustring>> var0 =
            Glib::Variant<std::vector<Glib::ustring>>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SystemDaemon
} // Kiran

