#pragma once
#include <string>
#include <tuple>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/touchpad_dbus_common.h"

namespace Kiran {
namespace SessionDaemon {

class TouchPadProxy : public Glib::ObjectBase {
public:
    static void createForBus(Gio::DBus::BusType busType,
                             Gio::DBus::ProxyFlags proxyFlags,
                             const std::string &name,
                             const std::string &objectPath,
                             const Gio::SlotAsyncReady &slot,
                             const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    static Glib::RefPtr<TouchPadProxy> createForBusFinish (const Glib::RefPtr<Gio::AsyncResult> &result);

    static Glib::RefPtr<TouchPadProxy> createForBus_sync(
        Gio::DBus::BusType busType,
        Gio::DBus::ProxyFlags proxyFlags,
        const std::string &name,
        const std::string &objectPath,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {});

    Glib::RefPtr<Gio::DBus::Proxy> dbusProxy() const { return m_proxy; }

    void Reset(
        const Gio::SlotAsyncReady &slot,
        const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    void Reset_finish (
        const Glib::RefPtr<Gio::AsyncResult> &res);

    void
    Reset_sync(
const Glib::RefPtr<Gio::Cancellable> &cancellable = {},
        int timeout_msec = -1);

    bool has_touchpad_get(bool *ok = nullptr);
    void has_touchpad_set(bool, const Gio::SlotAsyncReady &);
    void has_touchpad_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void has_touchpad_set_sync(bool);
    sigc::signal<void> &has_touchpad_changed() {
        return m_has_touchpad_changed;
    }

    bool left_handed_get(bool *ok = nullptr);
    void left_handed_set(bool, const Gio::SlotAsyncReady &);
    void left_handed_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void left_handed_set_sync(bool);
    sigc::signal<void> &left_handed_changed() {
        return m_left_handed_changed;
    }

    bool disable_while_typing_get(bool *ok = nullptr);
    void disable_while_typing_set(bool, const Gio::SlotAsyncReady &);
    void disable_while_typing_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void disable_while_typing_set_sync(bool);
    sigc::signal<void> &disable_while_typing_changed() {
        return m_disable_while_typing_changed;
    }

    bool tap_to_click_get(bool *ok = nullptr);
    void tap_to_click_set(bool, const Gio::SlotAsyncReady &);
    void tap_to_click_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void tap_to_click_set_sync(bool);
    sigc::signal<void> &tap_to_click_changed() {
        return m_tap_to_click_changed;
    }

    gint32 click_method_get(bool *ok = nullptr);
    void click_method_set(gint32, const Gio::SlotAsyncReady &);
    void click_method_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void click_method_set_sync(gint32);
    sigc::signal<void> &click_method_changed() {
        return m_click_method_changed;
    }

    gint32 scroll_method_get(bool *ok = nullptr);
    void scroll_method_set(gint32, const Gio::SlotAsyncReady &);
    void scroll_method_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void scroll_method_set_sync(gint32);
    sigc::signal<void> &scroll_method_changed() {
        return m_scroll_method_changed;
    }

    bool natural_scroll_get(bool *ok = nullptr);
    void natural_scroll_set(bool, const Gio::SlotAsyncReady &);
    void natural_scroll_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void natural_scroll_set_sync(bool);
    sigc::signal<void> &natural_scroll_changed() {
        return m_natural_scroll_changed;
    }

    bool touchpad_enabled_get(bool *ok = nullptr);
    void touchpad_enabled_set(bool, const Gio::SlotAsyncReady &);
    void touchpad_enabled_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void touchpad_enabled_set_sync(bool);
    sigc::signal<void> &touchpad_enabled_changed() {
        return m_touchpad_enabled_changed;
    }

    double motion_acceleration_get(bool *ok = nullptr);
    void motion_acceleration_set(double, const Gio::SlotAsyncReady &);
    void motion_acceleration_set_finish(const Glib::RefPtr<Gio::AsyncResult> &);
    void motion_acceleration_set_sync(double);
    sigc::signal<void> &motion_acceleration_changed() {
        return m_motion_acceleration_changed;
    }


    void reference() const override {}
    void unreference() const override {}

protected:
    Glib::RefPtr<Gio::DBus::Proxy> m_proxy;

private:
    TouchPadProxy(const Glib::RefPtr<Gio::DBus::Proxy> &proxy);

    void handle_signal(const Glib::ustring &sender_name,
                       const Glib::ustring &signal_name,
                       const Glib::VariantContainerBase &parameters);

    void handle_properties_changed(const Gio::DBus::Proxy::MapChangedProperties &changed_properties,
                                   const std::vector<Glib::ustring> &invalidated_properties);

    sigc::signal<void> m_has_touchpad_changed;
    sigc::signal<void> m_left_handed_changed;
    sigc::signal<void> m_disable_while_typing_changed;
    sigc::signal<void> m_tap_to_click_changed;
    sigc::signal<void> m_click_method_changed;
    sigc::signal<void> m_scroll_method_changed;
    sigc::signal<void> m_natural_scroll_changed;
    sigc::signal<void> m_touchpad_enabled_changed;
    sigc::signal<void> m_motion_acceleration_changed;
};

}// SessionDaemon
}// Kiran

