static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="com.kylinsec.Kiran.SessionDaemon.TouchPad">
        <method name="Reset">
            <description>Reset all properties to default value.</description>
        </method>

        <property name="has_touchpad" type="b" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>Whether has touchpad device.</description>
        </property>

        <property name="left_handed" type="b" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>Swap left and right mouse buttons for left-handed mice.</description>
        </property>

        <property name="disable_while_typing" type="b" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>Indicates if diable while typing is enabled or disabled..</description>
        </property>

        <property name="tap_to_click" type="b" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>Set this to TRUE to be able to send mouse clicks by tapping on the touchpad.</description>
        </property>

        <property name="click_method" type="i" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>The click method defines how button events are triggered on a clickpad. 
            When set to buttonareas, the bottom area of the touchpad is divided into a left, 
            middle and right button area. When set to clickfinger, the number of fingers on 
            the touchpad decide the button type. Clicking with 1, 2, 3 fingers triggers a left, 
            right, or middle click, respectively. The default click method is software button areas. 
            Click methods are usually only available on clickpads.</description>
        </property>

        <property name="scroll_method" type="i" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>Enables a scroll method. Permitted methods are twofinger, edge and button.</description>
        </property>

        <property name="natural_scroll" type="b" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description> "Natural scrolling" is the terminology for moving the content in the direction of scrolling, 
            i.e. moving the wheel or fingers down moves the page down. Traditional scrolling moves the content in the
            opposite direction. Natural scrolling can be turned on or off, it is usually off by default.
            </description>
        </property>

        <property name="touchpad_enabled" type="b" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>Set this to TRUE to enable all touchpads.</description>
        </property>

        <property name="motion_acceleration" type="d" access="readwrite">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="true"/>
            <description>Acceleration multiplier for mouse motion. The range of the value is [-1,1], the default value is 0.</description>
        </property>

    </interface>
</node>
)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/touchpad_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

Kiran::SessionDaemon::TouchPadStub::TouchPadStub():
    m_interfaceName("com.kylinsec.Kiran.SessionDaemon.TouchPad")
{
}

Kiran::SessionDaemon::TouchPadStub::~TouchPadStub()
{
    unregister_object();
}

guint Kiran::SessionDaemon::TouchPadStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &TouchPadStub::on_method_call),
            sigc::mem_fun(this, &TouchPadStub::on_interface_get_property),
            sigc::mem_fun(this, &TouchPadStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("com.kylinsec.Kiran.SessionDaemon.TouchPad"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void Kiran::SessionDaemon::TouchPadStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void Kiran::SessionDaemon::TouchPadStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("Reset") == 0) {
        MethodInvocation methodInvocation(invocation);
        Reset(
            methodInvocation);
    }

}

void Kiran::SessionDaemon::TouchPadStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

    if (property_name.compare("has_touchpad") == 0) {

        property = Glib::Variant<bool>::create((has_touchpad_get()));
    }

    if (property_name.compare("left_handed") == 0) {

        property = Glib::Variant<bool>::create((left_handed_get()));
    }

    if (property_name.compare("disable_while_typing") == 0) {

        property = Glib::Variant<bool>::create((disable_while_typing_get()));
    }

    if (property_name.compare("tap_to_click") == 0) {

        property = Glib::Variant<bool>::create((tap_to_click_get()));
    }

    if (property_name.compare("click_method") == 0) {

        property = Glib::Variant<gint32>::create((click_method_get()));
    }

    if (property_name.compare("scroll_method") == 0) {

        property = Glib::Variant<gint32>::create((scroll_method_get()));
    }

    if (property_name.compare("natural_scroll") == 0) {

        property = Glib::Variant<bool>::create((natural_scroll_get()));
    }

    if (property_name.compare("touchpad_enabled") == 0) {

        property = Glib::Variant<bool>::create((touchpad_enabled_get()));
    }

    if (property_name.compare("motion_acceleration") == 0) {

        property = Glib::Variant<double>::create((motion_acceleration_get()));
    }

}

bool Kiran::SessionDaemon::TouchPadStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    if (property_name.compare("has_touchpad") == 0) {
        try {
            Glib::Variant<bool> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(value);
            bool val =
                (specialGetter(castValue));
            has_touchpad_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting has_touchpad");
        }
    }

    if (property_name.compare("left_handed") == 0) {
        try {
            Glib::Variant<bool> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(value);
            bool val =
                (specialGetter(castValue));
            left_handed_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting left_handed");
        }
    }

    if (property_name.compare("disable_while_typing") == 0) {
        try {
            Glib::Variant<bool> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(value);
            bool val =
                (specialGetter(castValue));
            disable_while_typing_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting disable_while_typing");
        }
    }

    if (property_name.compare("tap_to_click") == 0) {
        try {
            Glib::Variant<bool> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(value);
            bool val =
                (specialGetter(castValue));
            tap_to_click_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting tap_to_click");
        }
    }

    if (property_name.compare("click_method") == 0) {
        try {
            Glib::Variant<gint32> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<gint32>>(value);
            gint32 val =
                (specialGetter(castValue));
            click_method_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting click_method");
        }
    }

    if (property_name.compare("scroll_method") == 0) {
        try {
            Glib::Variant<gint32> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<gint32>>(value);
            gint32 val =
                (specialGetter(castValue));
            scroll_method_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting scroll_method");
        }
    }

    if (property_name.compare("natural_scroll") == 0) {
        try {
            Glib::Variant<bool> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(value);
            bool val =
                (specialGetter(castValue));
            natural_scroll_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting natural_scroll");
        }
    }

    if (property_name.compare("touchpad_enabled") == 0) {
        try {
            Glib::Variant<bool> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(value);
            bool val =
                (specialGetter(castValue));
            touchpad_enabled_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting touchpad_enabled");
        }
    }

    if (property_name.compare("motion_acceleration") == 0) {
        try {
            Glib::Variant<double> castValue =
                Glib::VariantBase::cast_dynamic<Glib::Variant<double>>(value);
            double val =
                (specialGetter(castValue));
            motion_acceleration_set(val);
        } catch (const std::bad_cast &) {
            g_warning ("Bad cast when casting motion_acceleration");
        }
    }

    return true;
}


bool Kiran::SessionDaemon::TouchPadStub::has_touchpad_set(bool value)
{
    if (has_touchpad_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((has_touchpad_get()));

        emitSignal("has_touchpad", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::left_handed_set(bool value)
{
    if (left_handed_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((left_handed_get()));

        emitSignal("left_handed", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::disable_while_typing_set(bool value)
{
    if (disable_while_typing_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((disable_while_typing_get()));

        emitSignal("disable_while_typing", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::tap_to_click_set(bool value)
{
    if (tap_to_click_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((tap_to_click_get()));

        emitSignal("tap_to_click", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::click_method_set(gint32 value)
{
    if (click_method_setHandler(value)) {
        Glib::Variant<gint32> value_get =
            Glib::Variant<gint32>::create((click_method_get()));

        emitSignal("click_method", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::scroll_method_set(gint32 value)
{
    if (scroll_method_setHandler(value)) {
        Glib::Variant<gint32> value_get =
            Glib::Variant<gint32>::create((scroll_method_get()));

        emitSignal("scroll_method", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::natural_scroll_set(bool value)
{
    if (natural_scroll_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((natural_scroll_get()));

        emitSignal("natural_scroll", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::touchpad_enabled_set(bool value)
{
    if (touchpad_enabled_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((touchpad_enabled_get()));

        emitSignal("touchpad_enabled", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::motion_acceleration_set(double value)
{
    if (motion_acceleration_setHandler(value)) {
        Glib::Variant<double> value_get =
            Glib::Variant<double>::create((motion_acceleration_get()));

        emitSignal("motion_acceleration", value_get);
        return true;
    }

    return false;
}

bool Kiran::SessionDaemon::TouchPadStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
