static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="com.kylinsec.Kiran.SystemDaemon.Accounts.User">
        <method name="SetUserName">
            <arg name="name" direction="in" type="s">
                <summary>The new username.</summary>
            </arg>
            <description>Sets the users username. Note that it is usually not allowed to have multiple users with the same username.</description>
        </method>

        <method name="SetRealName">
            <arg name="name" direction="in" type="s">
                <description>The new name, typically in the form "Firstname Lastname".</description>
            </arg>
            <description>Sets the users real name.</description>
        </method>

        <method name="SetEmail">
            <arg name="email" direction="in" type="s">
                <summary>The new email address.</summary>
            </arg>
            <description>Sets the users email address.</description>
        </method>

        <method name="SetLanguage">
            <arg name="language" direction="in" type="s">
                <description>The new language, as a locale specification like "de_DE.UTF-8".</description>
            </arg>
            <description>Sets the users language.</description>
        </method>

        <method name="SetXSession">
            <arg name="x_session" direction="in" type="s">
                <description>The new xsession to start (e.g. "gnome")</description>
            </arg>
            <description>Sets the users x session.</description>
        </method>

        <method name="SetSession">
            <arg name="session" direction="in" type="s">
                <description>The new session to start (e.g. "gnome-xorg")</description>
            </arg>
            <description>Sets the users wayland or x session.</description>
        </method>

        <method name="SetSessionType">
            <arg name="session_type" direction="in" type="s">
                <description>The type of the new session to start (e.g. "wayland" or "x11")</description>
            </arg>
            <description>Sets the session type of the users session.</description>
        </method>

        <method name="SetHomeDirectory">
            <arg name="homedir" direction="in" type="s">
                <description>The new homedir as an absolute path.</description>
            </arg>
            <description>Sets the users home directory.</description>
        </method>

        <method name="SetShell">
            <arg name="shell" direction="in" type="s">
                <description>The new user shell.</description>
            </arg>
            <description>Sets the users shell.</description>
        </method>

        <method name="SetIconFile">
            <arg name="filename" direction="in" type="s">
                <description>The absolute filename of a png file to use as the users icon.</description>
            </arg>
            <description>Sets the users icon.</description>
        </method>

        <method name="SetLocked">
            <arg name="locked" direction="in" type="b">
                <description>Whether to lock or unlock the users account.</description>
            </arg>
            <description>Locks or unlocks a users account.</description>
        </method>

        <method name="SetAccountType">
            <arg name="account_type" direction="in" type="i">
                <description>The new account type, encoded as an integer. Refer to AccountsAccountType in accounts-i.h</description>
            </arg>
            <description>Changes the users account type.</description>
        </method>

        <method name="SetPasswordMode">
            <arg name="mode" direction="in" type="i">
                <description>The new password mode, encoded as an integer:</description>
            </arg>
            <description>Changes the users password mode. Note that changing the password mode has the side-effect of unlocking the account.</description>
        </method>

        <method name="SetPassword">
            <arg name="password" direction="in" type="s">
                <description>The crypted password.</description>
            </arg>
            <arg name="hint" direction="in" type="s">
                <description>The password hint.</description>
            </arg>
            <description>Sets a new password for this user. Note that setting a password has the side-effect of unlocking the account.</description>
        </method>

        <method name="SetPasswordHint">
            <arg name="hint" direction="in" type="s">
                <description>The password hint.</description>
            </arg>
            <description>Sets the users password hint.</description>
        </method>

        <method name="SetAutomaticLogin">
            <arg name="enabled" direction="in" type="b">
                <description>Whether to enable automatic login for this user.</description>
            </arg>
            <description>Enables or disables automatic login for a user.
            Note that usually only one user can have automatic login
            enabled, so turning it on for a user will disable it for
            the previously configured autologin user.</description>
        </method>

        <method name="SetPasswordExpirationPolicy">
            <arg name="options" direction="in" type="s">
                <description>The password expiration policy options that is a jason string which format refer to
                property password_expiration_policy.</description>
            </arg>
            <description>Set user password expiry information which contain expiration_time, last_change_time, min_days_between_changes,
            max_days_between_changes, days_to_warn, inactive_days_after_expiration</description>
        </method>

        <method name="AddAuthItem">
            <arg type="i" name="mode" direction="in">
                <description>The authentication mode which could be fingerprint, face, etc. Refer to AccountsAuthMode in accounts-i.h</description>
            </arg>
            <arg type="s" name="name" direction="in">
                <description>the name of the authentication item. For example fingerprint 1, fingerprint 2, etc.</description>
            </arg>
            <arg type="s" name="data_id" direction="in">
                <description>The identity of the collected data for the authentication item.</description>
            </arg>
            <description>Add an auth item for a mode. Return error if the name already exists or the method is unsupported.</description>
        </method>

        <method name="DelAuthItem">
            <arg type="i" name="mode" direction="in" />
            <arg type="s" name="name" direction="in" />
            <description>Delete an auth item by name. Return error if the name doesn't exist or the method is unsupported.</description>
        </method>

        <method name="GetAuthItems">
            <arg type="i" name="mode" direction="in" />
            <arg type="s" name="auth_items_json" direction="out">
                <description>The json string is an array that every item contains name and the identity of the collected data.</description>
            </arg>
        </method>

        <method name="EnableAuthMode">
            <arg type="i" name="mode" direction="in" />
            <arg type="b" name="enabled" direction="in" />
            <description>Enable or disable the authentication mode.</description>
        </method>

        <property name="uid" type="t" access="read">
            <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="false" />
            <description>The uid of the user.</description>
        </property>

        <property name="user_name" type="s" access="read">
            <description>The username of the user.</description>
        </property>

        <property name="real_name" type="s" access="read">
            <description>The users real name.</description>
        </property>

        <property name="account_type" type="i" access="read">
            <description>The users account type, encoded as an integer. Refer to AccountsAccountType in accounts-i.h</description>
        </property>

        <property name="home_directory" type="s" access="read">
            <description>The users home directory.</description>
        </property>

        <property name="shell" type="s" access="read">
            <description>The users shell.</description>
        </property>

        <property name="email" type="s" access="read">
            <description>The email address.</description>
        </property>

        <property name="language" type="s" access="read">
            <description>The users language, as a locale specification like "de_DE.UTF-8".</description>
        </property>

        <property name="session" type="s" access="read">
            <description>The users Wayland or X session.</description>
        </property>

        <property name="session_type" type="s" access="read">
            <description>The type of session the user should use (e.g. "wayland" or "x11")</description>
        </property>

        <property name="x_session" type="s" access="read">
            <description>The users x session.</description>
        </property>


        <property name="icon_file" type="s" access="read">
            <description>The filename of a png file containing the users icon.</description>
        </property>


        <property name="locked" type="b" access="read">
            <description>Whether the users account is locked.</description>
        </property>

        <property name="password_mode" type="i" access="read">
            <description>The password mode for the user account, encoded as an integer. Refer to AccountsPasswordMode in accounts-i.h</description>
        </property>

        <property name="password_hint" type="s" access="read">
            <description>The password hint for the user.</description>
        </property>

        <property name="automatic_login" type="b" access="read">
            <description>Whether automatic login is enabled for the user.</description>
        </property>

        <property name="system_account" type="b" access="read">
            <description>Whether this is a 'system' account, like 'root' or 'nobody'.
            System accounts should normally not appear in lists of users, and ListCachedUsers 
            will not include such accounts.</description>
        </property>

        <property name="auth_modes" type="i" access="read">
            <description>Bitwise OR the enabled authentication mode.</description>
        </property>

        <property name="password_expiration_policy" type="s" access="read">
            <description>The password expiration policy that is a jason string.</description>
        </property>

        <signal name="AuthItemChanged">
            <arg name="mode" type="i" />
            <description>Send this signal when the item of the authentication mode is added or deleted.</description>
        </signal>
    </interface>
</node>
)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/user_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

Kiran::SystemDaemon::Accounts::UserStub::UserStub():
    m_interfaceName("com.kylinsec.Kiran.SystemDaemon.Accounts.User")
{
    AuthItemChanged_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &UserStub::AuthItemChanged_emitter),
            std::vector<Glib::ustring>({""})) );
    AuthItemChanged_selectiveSignal.connect(sigc::mem_fun(this, &UserStub::AuthItemChanged_emitter));
}

Kiran::SystemDaemon::Accounts::UserStub::~UserStub()
{
    unregister_object();
}

guint Kiran::SystemDaemon::Accounts::UserStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &UserStub::on_method_call),
            sigc::mem_fun(this, &UserStub::on_interface_get_property),
            sigc::mem_fun(this, &UserStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("com.kylinsec.Kiran.SystemDaemon.Accounts.User"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void Kiran::SystemDaemon::Accounts::UserStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void Kiran::SystemDaemon::Accounts::UserStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("SetUserName") == 0) {
        Glib::Variant<Glib::ustring> base_name;
        parameters.get_child(base_name, 0);
        Glib::ustring p_name = specialGetter(base_name);

        MethodInvocation methodInvocation(invocation);
        SetUserName(
            (p_name),
            methodInvocation);
    }

    if (method_name.compare("SetRealName") == 0) {
        Glib::Variant<Glib::ustring> base_name;
        parameters.get_child(base_name, 0);
        Glib::ustring p_name = specialGetter(base_name);

        MethodInvocation methodInvocation(invocation);
        SetRealName(
            (p_name),
            methodInvocation);
    }

    if (method_name.compare("SetEmail") == 0) {
        Glib::Variant<Glib::ustring> base_email;
        parameters.get_child(base_email, 0);
        Glib::ustring p_email = specialGetter(base_email);

        MethodInvocation methodInvocation(invocation);
        SetEmail(
            (p_email),
            methodInvocation);
    }

    if (method_name.compare("SetLanguage") == 0) {
        Glib::Variant<Glib::ustring> base_language;
        parameters.get_child(base_language, 0);
        Glib::ustring p_language = specialGetter(base_language);

        MethodInvocation methodInvocation(invocation);
        SetLanguage(
            (p_language),
            methodInvocation);
    }

    if (method_name.compare("SetXSession") == 0) {
        Glib::Variant<Glib::ustring> base_x_session;
        parameters.get_child(base_x_session, 0);
        Glib::ustring p_x_session = specialGetter(base_x_session);

        MethodInvocation methodInvocation(invocation);
        SetXSession(
            (p_x_session),
            methodInvocation);
    }

    if (method_name.compare("SetSession") == 0) {
        Glib::Variant<Glib::ustring> base_session;
        parameters.get_child(base_session, 0);
        Glib::ustring p_session = specialGetter(base_session);

        MethodInvocation methodInvocation(invocation);
        SetSession(
            (p_session),
            methodInvocation);
    }

    if (method_name.compare("SetSessionType") == 0) {
        Glib::Variant<Glib::ustring> base_session_type;
        parameters.get_child(base_session_type, 0);
        Glib::ustring p_session_type = specialGetter(base_session_type);

        MethodInvocation methodInvocation(invocation);
        SetSessionType(
            (p_session_type),
            methodInvocation);
    }

    if (method_name.compare("SetHomeDirectory") == 0) {
        Glib::Variant<Glib::ustring> base_homedir;
        parameters.get_child(base_homedir, 0);
        Glib::ustring p_homedir = specialGetter(base_homedir);

        MethodInvocation methodInvocation(invocation);
        SetHomeDirectory(
            (p_homedir),
            methodInvocation);
    }

    if (method_name.compare("SetShell") == 0) {
        Glib::Variant<Glib::ustring> base_shell;
        parameters.get_child(base_shell, 0);
        Glib::ustring p_shell = specialGetter(base_shell);

        MethodInvocation methodInvocation(invocation);
        SetShell(
            (p_shell),
            methodInvocation);
    }

    if (method_name.compare("SetIconFile") == 0) {
        Glib::Variant<Glib::ustring> base_filename;
        parameters.get_child(base_filename, 0);
        Glib::ustring p_filename = specialGetter(base_filename);

        MethodInvocation methodInvocation(invocation);
        SetIconFile(
            (p_filename),
            methodInvocation);
    }

    if (method_name.compare("SetLocked") == 0) {
        Glib::Variant<bool> base_locked;
        parameters.get_child(base_locked, 0);
        bool p_locked = specialGetter(base_locked);

        MethodInvocation methodInvocation(invocation);
        SetLocked(
            (p_locked),
            methodInvocation);
    }

    if (method_name.compare("SetAccountType") == 0) {
        Glib::Variant<gint32> base_account_type;
        parameters.get_child(base_account_type, 0);
        gint32 p_account_type = specialGetter(base_account_type);

        MethodInvocation methodInvocation(invocation);
        SetAccountType(
            (p_account_type),
            methodInvocation);
    }

    if (method_name.compare("SetPasswordMode") == 0) {
        Glib::Variant<gint32> base_mode;
        parameters.get_child(base_mode, 0);
        gint32 p_mode = specialGetter(base_mode);

        MethodInvocation methodInvocation(invocation);
        SetPasswordMode(
            (p_mode),
            methodInvocation);
    }

    if (method_name.compare("SetPassword") == 0) {
        Glib::Variant<Glib::ustring> base_password;
        parameters.get_child(base_password, 0);
        Glib::ustring p_password = specialGetter(base_password);

        Glib::Variant<Glib::ustring> base_hint;
        parameters.get_child(base_hint, 1);
        Glib::ustring p_hint = specialGetter(base_hint);

        MethodInvocation methodInvocation(invocation);
        SetPassword(
            (p_password),
            (p_hint),
            methodInvocation);
    }

    if (method_name.compare("SetPasswordHint") == 0) {
        Glib::Variant<Glib::ustring> base_hint;
        parameters.get_child(base_hint, 0);
        Glib::ustring p_hint = specialGetter(base_hint);

        MethodInvocation methodInvocation(invocation);
        SetPasswordHint(
            (p_hint),
            methodInvocation);
    }

    if (method_name.compare("SetAutomaticLogin") == 0) {
        Glib::Variant<bool> base_enabled;
        parameters.get_child(base_enabled, 0);
        bool p_enabled = specialGetter(base_enabled);

        MethodInvocation methodInvocation(invocation);
        SetAutomaticLogin(
            (p_enabled),
            methodInvocation);
    }

    if (method_name.compare("SetPasswordExpirationPolicy") == 0) {
        Glib::Variant<Glib::ustring> base_options;
        parameters.get_child(base_options, 0);
        Glib::ustring p_options = specialGetter(base_options);

        MethodInvocation methodInvocation(invocation);
        SetPasswordExpirationPolicy(
            (p_options),
            methodInvocation);
    }

    if (method_name.compare("AddAuthItem") == 0) {
        Glib::Variant<gint32> base_mode;
        parameters.get_child(base_mode, 0);
        gint32 p_mode = specialGetter(base_mode);

        Glib::Variant<Glib::ustring> base_name;
        parameters.get_child(base_name, 1);
        Glib::ustring p_name = specialGetter(base_name);

        Glib::Variant<Glib::ustring> base_data_id;
        parameters.get_child(base_data_id, 2);
        Glib::ustring p_data_id = specialGetter(base_data_id);

        MethodInvocation methodInvocation(invocation);
        AddAuthItem(
            (p_mode),
            (p_name),
            (p_data_id),
            methodInvocation);
    }

    if (method_name.compare("DelAuthItem") == 0) {
        Glib::Variant<gint32> base_mode;
        parameters.get_child(base_mode, 0);
        gint32 p_mode = specialGetter(base_mode);

        Glib::Variant<Glib::ustring> base_name;
        parameters.get_child(base_name, 1);
        Glib::ustring p_name = specialGetter(base_name);

        MethodInvocation methodInvocation(invocation);
        DelAuthItem(
            (p_mode),
            (p_name),
            methodInvocation);
    }

    if (method_name.compare("GetAuthItems") == 0) {
        Glib::Variant<gint32> base_mode;
        parameters.get_child(base_mode, 0);
        gint32 p_mode = specialGetter(base_mode);

        MethodInvocation methodInvocation(invocation);
        GetAuthItems(
            (p_mode),
            methodInvocation);
    }

    if (method_name.compare("EnableAuthMode") == 0) {
        Glib::Variant<gint32> base_mode;
        parameters.get_child(base_mode, 0);
        gint32 p_mode = specialGetter(base_mode);

        Glib::Variant<bool> base_enabled;
        parameters.get_child(base_enabled, 1);
        bool p_enabled = specialGetter(base_enabled);

        MethodInvocation methodInvocation(invocation);
        EnableAuthMode(
            (p_mode),
            (p_enabled),
            methodInvocation);
    }

}

void Kiran::SystemDaemon::Accounts::UserStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

    if (property_name.compare("uid") == 0) {

        property = Glib::Variant<guint64>::create((uid_get()));
    }

    if (property_name.compare("user_name") == 0) {

        property = Glib::Variant<Glib::ustring>::create((user_name_get()));
    }

    if (property_name.compare("real_name") == 0) {

        property = Glib::Variant<Glib::ustring>::create((real_name_get()));
    }

    if (property_name.compare("account_type") == 0) {

        property = Glib::Variant<gint32>::create((account_type_get()));
    }

    if (property_name.compare("home_directory") == 0) {

        property = Glib::Variant<Glib::ustring>::create((home_directory_get()));
    }

    if (property_name.compare("shell") == 0) {

        property = Glib::Variant<Glib::ustring>::create((shell_get()));
    }

    if (property_name.compare("email") == 0) {

        property = Glib::Variant<Glib::ustring>::create((email_get()));
    }

    if (property_name.compare("language") == 0) {

        property = Glib::Variant<Glib::ustring>::create((language_get()));
    }

    if (property_name.compare("session") == 0) {

        property = Glib::Variant<Glib::ustring>::create((session_get()));
    }

    if (property_name.compare("session_type") == 0) {

        property = Glib::Variant<Glib::ustring>::create((session_type_get()));
    }

    if (property_name.compare("x_session") == 0) {

        property = Glib::Variant<Glib::ustring>::create((x_session_get()));
    }

    if (property_name.compare("icon_file") == 0) {

        property = Glib::Variant<Glib::ustring>::create((icon_file_get()));
    }

    if (property_name.compare("locked") == 0) {

        property = Glib::Variant<bool>::create((locked_get()));
    }

    if (property_name.compare("password_mode") == 0) {

        property = Glib::Variant<gint32>::create((password_mode_get()));
    }

    if (property_name.compare("password_hint") == 0) {

        property = Glib::Variant<Glib::ustring>::create((password_hint_get()));
    }

    if (property_name.compare("automatic_login") == 0) {

        property = Glib::Variant<bool>::create((automatic_login_get()));
    }

    if (property_name.compare("system_account") == 0) {

        property = Glib::Variant<bool>::create((system_account_get()));
    }

    if (property_name.compare("auth_modes") == 0) {

        property = Glib::Variant<gint32>::create((auth_modes_get()));
    }

    if (property_name.compare("password_expiration_policy") == 0) {

        property = Glib::Variant<Glib::ustring>::create((password_expiration_policy_get()));
    }

}

bool Kiran::SystemDaemon::Accounts::UserStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    return true;
}

void Kiran::SystemDaemon::Accounts::UserStub::AuthItemChanged_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,gint32 mode)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<gint32>::create((mode)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "com.kylinsec.Kiran.SystemDaemon.Accounts.User",
                    "AuthItemChanged",
                    bus_name,
                    params);
        }
    }
}


bool Kiran::SystemDaemon::Accounts::UserStub::uid_set(guint64 value)
{
    if (uid_setHandler(value)) {
        Glib::Variant<guint64> value_get =
            Glib::Variant<guint64>::create((uid_get()));

        emitSignal("uid", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::user_name_set(const Glib::ustring & value)
{
    if (user_name_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((user_name_get()));

        emitSignal("user_name", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::real_name_set(const Glib::ustring & value)
{
    if (real_name_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((real_name_get()));

        emitSignal("real_name", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::account_type_set(gint32 value)
{
    if (account_type_setHandler(value)) {
        Glib::Variant<gint32> value_get =
            Glib::Variant<gint32>::create((account_type_get()));

        emitSignal("account_type", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::home_directory_set(const Glib::ustring & value)
{
    if (home_directory_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((home_directory_get()));

        emitSignal("home_directory", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::shell_set(const Glib::ustring & value)
{
    if (shell_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((shell_get()));

        emitSignal("shell", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::email_set(const Glib::ustring & value)
{
    if (email_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((email_get()));

        emitSignal("email", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::language_set(const Glib::ustring & value)
{
    if (language_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((language_get()));

        emitSignal("language", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::session_set(const Glib::ustring & value)
{
    if (session_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((session_get()));

        emitSignal("session", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::session_type_set(const Glib::ustring & value)
{
    if (session_type_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((session_type_get()));

        emitSignal("session_type", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::x_session_set(const Glib::ustring & value)
{
    if (x_session_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((x_session_get()));

        emitSignal("x_session", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::icon_file_set(const Glib::ustring & value)
{
    if (icon_file_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((icon_file_get()));

        emitSignal("icon_file", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::locked_set(bool value)
{
    if (locked_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((locked_get()));

        emitSignal("locked", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::password_mode_set(gint32 value)
{
    if (password_mode_setHandler(value)) {
        Glib::Variant<gint32> value_get =
            Glib::Variant<gint32>::create((password_mode_get()));

        emitSignal("password_mode", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::password_hint_set(const Glib::ustring & value)
{
    if (password_hint_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((password_hint_get()));

        emitSignal("password_hint", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::automatic_login_set(bool value)
{
    if (automatic_login_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((automatic_login_get()));

        emitSignal("automatic_login", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::system_account_set(bool value)
{
    if (system_account_setHandler(value)) {
        Glib::Variant<bool> value_get =
            Glib::Variant<bool>::create((system_account_get()));

        emitSignal("system_account", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::auth_modes_set(gint32 value)
{
    if (auth_modes_setHandler(value)) {
        Glib::Variant<gint32> value_get =
            Glib::Variant<gint32>::create((auth_modes_get()));

        emitSignal("auth_modes", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::password_expiration_policy_set(const Glib::ustring & value)
{
    if (password_expiration_policy_setHandler(value)) {
        Glib::Variant<Glib::ustring> value_get =
            Glib::Variant<Glib::ustring>::create((password_expiration_policy_get()));

        emitSignal("password_expiration_policy", value_get);
        return true;
    }

    return false;
}

bool Kiran::SystemDaemon::Accounts::UserStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
