#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-cc-daemon-2.1.0/generated/user_dbus_common.h"

namespace Kiran {
namespace SystemDaemon {
namespace Accounts {

class UserStub : public sigc::trackable {
public:
    UserStub();
    virtual ~UserStub();

    UserStub(const UserStub &other) = delete;
    UserStub(UserStub &&other) = delete;
    UserStub &operator=(const UserStub &other) = delete;
    UserStub &operator=(UserStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool uid_set(guint64 value);
    bool user_name_set(const Glib::ustring & value);
    bool real_name_set(const Glib::ustring & value);
    bool account_type_set(gint32 value);
    bool home_directory_set(const Glib::ustring & value);
    bool shell_set(const Glib::ustring & value);
    bool email_set(const Glib::ustring & value);
    bool language_set(const Glib::ustring & value);
    bool session_set(const Glib::ustring & value);
    bool session_type_set(const Glib::ustring & value);
    bool x_session_set(const Glib::ustring & value);
    bool icon_file_set(const Glib::ustring & value);
    bool locked_set(bool value);
    bool password_mode_set(gint32 value);
    bool password_hint_set(const Glib::ustring & value);
    bool automatic_login_set(bool value);
    bool system_account_set(bool value);
    bool auth_modes_set(gint32 value);
    bool password_expiration_policy_set(const Glib::ustring & value);
protected:
    virtual void SetUserName(
        const Glib::ustring & name,
        MethodInvocation &invocation) = 0;
    virtual void SetRealName(
        const Glib::ustring & name,
        MethodInvocation &invocation) = 0;
    virtual void SetEmail(
        const Glib::ustring & email,
        MethodInvocation &invocation) = 0;
    virtual void SetLanguage(
        const Glib::ustring & language,
        MethodInvocation &invocation) = 0;
    virtual void SetXSession(
        const Glib::ustring & x_session,
        MethodInvocation &invocation) = 0;
    virtual void SetSession(
        const Glib::ustring & session,
        MethodInvocation &invocation) = 0;
    virtual void SetSessionType(
        const Glib::ustring & session_type,
        MethodInvocation &invocation) = 0;
    virtual void SetHomeDirectory(
        const Glib::ustring & homedir,
        MethodInvocation &invocation) = 0;
    virtual void SetShell(
        const Glib::ustring & shell,
        MethodInvocation &invocation) = 0;
    virtual void SetIconFile(
        const Glib::ustring & filename,
        MethodInvocation &invocation) = 0;
    virtual void SetLocked(
        bool locked,
        MethodInvocation &invocation) = 0;
    virtual void SetAccountType(
        gint32 account_type,
        MethodInvocation &invocation) = 0;
    virtual void SetPasswordMode(
        gint32 mode,
        MethodInvocation &invocation) = 0;
    virtual void SetPassword(
        const Glib::ustring & password,
        const Glib::ustring & hint,
        MethodInvocation &invocation) = 0;
    virtual void SetPasswordHint(
        const Glib::ustring & hint,
        MethodInvocation &invocation) = 0;
    virtual void SetAutomaticLogin(
        bool enabled,
        MethodInvocation &invocation) = 0;
    virtual void SetPasswordExpirationPolicy(
        const Glib::ustring & options,
        MethodInvocation &invocation) = 0;
    virtual void AddAuthItem(
        gint32 mode,
        const Glib::ustring & name,
        const Glib::ustring & data_id,
        MethodInvocation &invocation) = 0;
    virtual void DelAuthItem(
        gint32 mode,
        const Glib::ustring & name,
        MethodInvocation &invocation) = 0;
    virtual void GetAuthItems(
        gint32 mode,
        MethodInvocation &invocation) = 0;
    virtual void EnableAuthMode(
        gint32 mode,
        bool enabled,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool uid_setHandler(guint64 value) = 0;
    virtual guint64 uid_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool user_name_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring user_name_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool real_name_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring real_name_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool account_type_setHandler(gint32 value) = 0;
    virtual gint32 account_type_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool home_directory_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring home_directory_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool shell_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring shell_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool email_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring email_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool language_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring language_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool session_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring session_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool session_type_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring session_type_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool x_session_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring x_session_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool icon_file_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring icon_file_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool locked_setHandler(bool value) = 0;
    virtual bool locked_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool password_mode_setHandler(gint32 value) = 0;
    virtual gint32 password_mode_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool password_hint_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring password_hint_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool automatic_login_setHandler(bool value) = 0;
    virtual bool automatic_login_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool system_account_setHandler(bool value) = 0;
    virtual bool system_account_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool auth_modes_setHandler(gint32 value) = 0;
    virtual gint32 auth_modes_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool password_expiration_policy_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring password_expiration_policy_get() = 0;

    void AuthItemChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, gint32);
    sigc::signal<void, gint32> AuthItemChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, gint32> AuthItemChanged_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class UserStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

    void ret(const Glib::ustring & p0) {
        std::vector<Glib::VariantBase> vlist;
        Glib::Variant<Glib::ustring> var0 =
            Glib::Variant<Glib::ustring>::create(p0);
        vlist.push_back(var0);

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // Accounts
} // SystemDaemon
} // Kiran

