#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace Kiran {
namespace SessionDaemon {

class XSettingsTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase GetInteger_pack(
        const Glib::ustring & arg_name) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_name);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetInteger_pack(
        const Glib::ustring & arg_name,
        gint32 arg_value) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> name_param =
            Glib::Variant<Glib::ustring>::create(arg_name);
        params.push_back(name_param);

        Glib::Variant<gint32> value_param =
            Glib::Variant<gint32>::create(arg_value);
        params.push_back(value_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetString_pack(
        const Glib::ustring & arg_name) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_name);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetString_pack(
        const Glib::ustring & arg_name,
        const Glib::ustring & arg_value) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> name_param =
            Glib::Variant<Glib::ustring>::create(arg_name);
        params.push_back(name_param);

        Glib::Variant<Glib::ustring> value_param =
            Glib::Variant<Glib::ustring>::create(arg_value);
        params.push_back(value_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetColor_pack(
        const Glib::ustring & arg_name) {
        Glib::VariantContainerBase base;
        Glib::Variant<Glib::ustring> params =
            Glib::Variant<Glib::ustring>::create(arg_name);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase SetColor_pack(
        const Glib::ustring & arg_name,
        const std::tuple<guint16,guint16,guint16,guint16> & arg_value) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> name_param =
            Glib::Variant<Glib::ustring>::create(arg_name);
        params.push_back(name_param);

        Glib::Variant<std::tuple<guint16,guint16,guint16,guint16>> value_param =
            Glib::Variant<std::tuple<guint16,guint16,guint16,guint16>>::create(arg_value);
        params.push_back(value_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // SessionDaemon
} // Kiran


